@@ -173,7 +173,7 @@ $GREEN  --branch              $NO_COLOR Git branch name, tag (Rally release), co
 $GREEN  -f, --overwrite       $NO_COLOR Deprecated. Use -r instead.
 $GREEN  -r, --recreate        $NO_COLOR Remove target directory if it already exist.
                          If neither '-r' nor '-R' is set default behaviour is to ask.
-$GREEN  -R, --no-recreate     $NO_COLOR Do not reemove target directory if it already exist.
+$GREEN  -R, --no-recreate     $NO_COLOR Do not remove target directory if it already exist.
                          If neither '-r' nor '-R' is set default behaviour is to ask.
 $GREEN  -y, --yes             $NO_COLOR Do not ask for confirmation: assume a 'yes' reply
                          to every question.
@@ -347,7 +347,7 @@ __EOF__
                     missing=${missing//python-pip/}
                     if ! $pkg_manager python-pip; then
                         if ask_yn "Error installing python-pip. Install from external source?"; then
-                            local pdir=$(mktemp -d)
+                            local pdir=$(mktemp /tmp/tmp.XXXXXXXXXX -d)
                             local getpip="$pdir/get-pip.py"
                             download "$getpip" https://bootstrap.pypa.io/get-pip.py
                             if ! "$PYTHON" "$getpip"; then
@@ -460,7 +460,7 @@ setup_rally_configuration () {
     cp "$SRCDIR"/etc/rally/rally.conf.sample "$ETCDIR"/rally.conf
 
     [ -d "$DBDIR" ] || mkdir -p "$DBDIR"
-    local CONF_TMPFILE=$(mktemp)
+    local CONF_TMPFILE=$(mktemp /tmp/tmp.XXXXXXXXXX)
     sed "s|#connection *=.*|connection = \"$DBCONNSTRING\"|" "$ETCDIR"/rally.conf > "$CONF_TMPFILE"
     cat "$CONF_TMPFILE" > "$ETCDIR"/rally.conf
     rm "$CONF_TMPFILE"
