@@ -26,6 +26,7 @@ import jsonschema
 from oslo_config import cfg
 from oslo_utils import encodeutils
 import prettytable
+from requests.packages import urllib3
 import six
 import sqlalchemy.exc
 
@@ -79,7 +80,7 @@ def validate_args(fn, *args, **kwargs):
     MissingArgs: Missing argument(s): b, d
 
     :param fn: the function to check
-    :param arg: the positional arguments supplied
+    :param args: the positional arguments supplied
     :param kwargs: the keyword arguments supplied
     """
     argspec = inspect.getargspec(fn)
@@ -87,14 +88,11 @@ def validate_args(fn, *args, **kwargs):
     num_defaults = len(argspec.defaults or [])
     required_args = argspec.args[:len(argspec.args) - num_defaults]
 
-    def isbound(method):
-        return getattr(method, "__self__", None) is not None
-
-    if isbound(fn):
+    if getattr(fn, "__self__", None):
         required_args.pop(0)
 
-    missing = [arg for arg in required_args if arg not in kwargs]
-    missing = missing[len(args):]
+    missing_required_args = required_args[len(args):]
+    missing = [arg for arg in missing_required_args if arg not in kwargs]
     if missing:
         raise MissingArgs(missing)
 
@@ -138,6 +136,8 @@ def print_list(objs, fields, formatters=None, sortby_index=0,
         for field in fields:
             if field in formatters:
                 row.append(formatters[field](o))
+            elif type(o) == dict:
+                row.append(o[field])
             else:
                 if field in mixed_case_fields:
                     field_name = field.replace(" ", "_")
@@ -185,7 +185,7 @@ def make_table_header(table_label, table_width,
     :param horizontal_char: character used for horizontal lines.
     :param vertical_char: character used for vertical lines.
 
-    :returns string
+    :returns: string
     """
 
     if len(table_label) >= (table_width - 2):
@@ -282,33 +282,41 @@ class CategoryParser(argparse.ArgumentParser):
 
     def error(self, message):
         self.print_help(sys.stderr)
+        if message.startswith("argument") and message.endswith("is required"):
+            # NOTE(pirsriva) Argparse will currently raise an error
+            # message for only 1 missing argument at a time i.e. in the
+            # error message it WILL NOT LIST ALL the missing arguments
+            # at once INSTEAD only 1 missing argument at a time
+            missing_arg = message.split()[1]
+            print(_("Missing argument:\n%s") % missing_arg)
         sys.exit(2)
 
 
 def pretty_float_formatter(field, ndigits=None):
-    """Create a formatter function for the given float field.
+    """Create a float value formatter function for the given field.
 
-    :param field: a float object attribute name to be formatted.
-    :param ndigits: The number of digits after decimal point after round.
-    If None, then no rounding will be done.
-    :returns: the formatter function
+    :param field: str name of an object, which value should be formatted
+    :param ndigits: int number of digits after decimal point to round
+                    default is None - this disables rounding
+    :returns: field formatter function
     """
-
     def _formatter(obj):
-        value = getattr(obj, field)
-        if value is not None:
-            if ndigits is not None:
+        value = obj[field] if type(obj) == dict else getattr(obj, field)
+        if type(value) in (int, float):
+            if ndigits:
                 return round(value, ndigits)
-            else:
-                return value
-        else:
-            return "n/a"
+            return value
+        return "n/a"
     return _formatter
 
 
 def args(*args, **kwargs):
     def _decorator(func):
         func.__dict__.setdefault("args", []).insert(0, (args, kwargs))
+        if "metavar" not in kwargs and "action" not in kwargs:
+            # NOTE(andreykurilin): argparse constructs awful metavars...
+            kwargs["metavar"] = "<%s>" % args[0].replace(
+                "--", "").replace("-", "_")
         return func
     return _decorator
 
@@ -326,12 +334,19 @@ def alias(command_name):
 
 def deprecated_args(*args, **kwargs):
     def _decorator(func):
+        if "release" not in kwargs:
+            raise ValueError("'release' is required keyword argument of "
+                             "'deprecated_args' decorator.")
         func.__dict__.setdefault("args", []).insert(0, (args, kwargs))
         func.__dict__.setdefault("deprecated_args", [])
         func.deprecated_args.append(args[0])
-        if "help" in kwargs.keys():
-            warn_message = "DEPRECATED!"
-            kwargs["help"] = " ".join([warn_message, kwargs["help"]])
+
+        help_msg = "[Deprecated since Rally %s] " % kwargs.pop("release")
+        if "alternative" in kwargs:
+            help_msg += "Use '%s' instead. " % kwargs.pop("alternative")
+        if "help" in kwargs:
+            help_msg += kwargs["help"]
+        kwargs["help"] = help_msg
         return func
     return _decorator
 
@@ -477,6 +492,12 @@ def run(argv, categories):
             urllib3_log = logging.getLogger("urllib3").logger
             urllib3_log.setLevel(logging.WARNING)
 
+            LOG.debug("urllib3 insecure warnings are hidden.")
+            urllib3.disable_warnings(
+                urllib3.exceptions.InsecurePlatformWarning)
+            urllib3.disable_warnings(urllib3.exceptions.SNIMissingWarning)
+            urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
+
             # NOTE(wtakase): This is for suppressing boto error logging.
             LOG.debug("ERROR log from boto module is hide.")
             boto_log = logging.getLogger("boto").logger
