@@ -19,11 +19,13 @@ from __future__ import print_function
 import json
 import os
 import sys
+import traceback
 import webbrowser
 
 import jsonschema
 from oslo_utils import uuidutils
 import six
+from six.moves.urllib import parse as urlparse
 import yaml
 
 from rally import api
@@ -37,13 +39,15 @@ from rally.common import utils as rutils
 from rally import consts
 from rally import exceptions
 from rally import plugins
+from rally.task import exporter
 from rally.task.processing import plot
-from rally.task.processing import utils
 
 
 class FailedToLoadTask(exceptions.RallyException):
     msg_fmt = _("Failed to load task")
 
+LOG = logging.getLogger(__name__)
+
 
 class TaskCommands(object):
     """Set of commands that allow you to manage benchmarking tasks and results.
@@ -128,10 +132,14 @@ class TaskCommands(object):
 
     def _load_and_validate_task(self, task, task_args, task_args_file,
                                 deployment, task_instance=None):
-        if not os.path.exists(task) or os.path.isdir(task):
+        if not os.path.isfile(task):
+            err_cls = IOError
+            msg = "No such file '%s'" % task
             if task_instance:
-                task_instance.set_failed(log="No such file '%s'" % task)
-            raise IOError("File '%s' is not found." % task)
+                task_instance.set_failed(err_cls.__name__,
+                                         msg,
+                                         json.dumps(traceback.format_stack()))
+            raise err_cls(msg)
         input_task = self._load_task(task, task_args, task_args_file)
         api.Task.validate(deployment, input_task, task_instance)
         print(_("Task config is valid :)"))
@@ -249,7 +257,9 @@ class TaskCommands(object):
             self.detailed(task_id=task_instance["uuid"])
 
         except (exceptions.InvalidTaskException, FailedToLoadTask) as e:
-            task_instance.set_failed(log=e.format_message())
+            task_instance.set_failed(type(e).__name__,
+                                     str(e),
+                                     json.dumps(traceback.format_exc()))
             print(e, file=sys.stderr)
             return(1)
 
@@ -296,69 +306,41 @@ class TaskCommands(object):
                    help="Print detailed results for each iteration.")
     @envutils.with_default_task_id
     def detailed(self, task_id=None, iterations_data=False):
-        """Display results table.
+        """Print detailed information about given task.
 
-        :param task_id: Task uuid
-        :param iterations_data: print detailed results for each iteration
-        Prints detailed information of task.
+        :param task_id: str, task uuid
+        :param iterations_data: bool, include results for each iteration
         """
+        task = api.Task.get_detailed(task_id, extended_results=True)
 
-        def _print_iterations_data(result):
-            raw_data = result["data"]["raw"]
-            headers = ["iteration", "full duration"]
-            float_cols = ["full duration"]
-            atomic_actions = []
-            for row in raw_data:
-                # find first non-error result to get atomic actions names
-                if not row["error"] and "atomic_actions" in row:
-                    atomic_actions = row["atomic_actions"].keys()
-            for row in raw_data:
-                if row["atomic_actions"]:
-                    for (c, a) in enumerate(atomic_actions, 1):
-                        action = "%(no)i. %(action)s" % {"no": c, "action": a}
-                        headers.append(action)
-                        float_cols.append(action)
-                    break
-            table_rows = []
-            formatters = dict(zip(float_cols,
-                                  [cliutils.pretty_float_formatter(col, 3)
-                                   for col in float_cols]))
-            for (c, r) in enumerate(raw_data, 1):
-                dlist = [c]
-                dlist.append(r["duration"])
-                if r["atomic_actions"]:
-                    for action in atomic_actions:
-                        dlist.append(r["atomic_actions"].get(action) or 0)
-                table_rows.append(rutils.Struct(**dict(zip(headers,
-                                                           dlist))))
-            cliutils.print_list(table_rows,
-                                fields=headers,
-                                formatters=formatters)
-            print()
+        if not task:
+            print("The task %s can not be found" % task_id)
+            return 1
 
-        def _print_task_info(task):
-            print()
+        print()
+        print("-" * 80)
+        print(_("Task %(task_id)s: %(status)s")
+              % {"task_id": task_id, "status": task["status"]})
+
+        if task["status"] == consts.TaskStatus.FAILED:
             print("-" * 80)
-            print(_("Task %(task_id)s: %(status)s")
-                  % {"task_id": task_id, "status": task["status"]})
-
-            if task["status"] == consts.TaskStatus.FAILED:
-                print("-" * 80)
-                verification = yaml.safe_load(task["verification_log"])
-
-                if not logging.is_debug():
-                    print(verification[0])
-                    print(verification[1])
-                    print()
-                    print(_("For more details run:\n"
-                            "rally -vd task detailed %s")
-                          % task["uuid"])
-                else:
-                    print(yaml.safe_load(verification[2]))
-                return False
-            return True
-
-        def _print_scenario_args(result):
+            verification = yaml.safe_load(task["verification_log"])
+            if logging.is_debug():
+                print(yaml.safe_load(verification[2]))
+            else:
+                print(verification[0])
+                print(verification[1])
+                print(_("\nFor more details run:\nrally -vd task detailed %s")
+                      % task["uuid"])
+            return 0
+        elif task["status"] not in [consts.TaskStatus.FINISHED,
+                                    consts.TaskStatus.ABORTED]:
+            print("-" * 80)
+            print(_("\nThe task %s marked as '%s'. Results "
+                    "available when it is '%s'.") % (
+                task_id, task["status"], consts.TaskStatus.FINISHED))
+            return 0
+        for result in task["results"]:
             key = result["key"]
             print("-" * 80)
             print()
@@ -366,145 +348,112 @@ class TaskCommands(object):
             print("args position %s" % key["pos"])
             print("args values:")
             print(json.dumps(key["kw"], indent=2))
+            print()
 
-        def _print_summrized_result(result):
-            raw = result["data"]["raw"]
-            table_cols = ["action", "min", "median",
-                          "90%ile", "95%ile", "max",
-                          "avg", "success", "count"]
-            float_cols = ["min", "median",
-                          "90%ile", "95%ile", "max",
-                          "avg"]
-            formatters = dict(zip(float_cols,
-                                  [cliutils.pretty_float_formatter(col, 3)
-                                   for col in float_cols]))
-            table_rows = []
+            durations = plot.charts.MainStatsTable(result["info"])
+            iterations = []
+            iterations_headers = ["iteration", "full duration"]
+            iterations_actions = []
+            output = []
+            task_errors = []
+            if iterations_data:
+                for i, atomic_name in enumerate(result["info"]["atomic"], 1):
+                    action = "%i. %s" % (i, atomic_name)
+                    iterations_headers.append(action)
+                    iterations_actions.append((atomic_name, action))
 
-            actions_data = utils.get_atomic_actions_data(raw)
-            for action in actions_data:
-                durations = actions_data[action]
-                if durations:
-                    data = [action,
-                            round(min(durations), 3),
-                            round(utils.median(durations), 3),
-                            round(utils.percentile(durations, 0.90), 3),
-                            round(utils.percentile(durations, 0.95), 3),
-                            round(max(durations), 3),
-                            round(utils.mean(durations), 3),
-                            "%.1f%%" % (len(durations) * 100.0 / len(raw)),
-                            len(raw)]
+            for idx, itr in enumerate(result["iterations"], 1):
+                durations.add_iteration(itr)
+
+                if iterations_data:
+                    row = {"iteration": idx,
+                           "full duration": itr["duration"]}
+                    for name, action in iterations_actions:
+                        row[action] = itr["atomic_actions"].get(name, 0)
+                    iterations.append(row)
+
+                if "output" in itr:
+                    iteration_output = itr["output"]
                 else:
-                    data = [action, None, None, None, None, None, None,
-                            "0.0%", len(raw)]
-                table_rows.append(rutils.Struct(**dict(zip(table_cols,
-                                                           data))))
-
-            cliutils.print_list(table_rows, fields=table_cols,
-                                formatters=formatters,
-                                table_label="Response Times (sec)",
-                                sortby_index=None)
-
-        def _print_ssrs_result(result):
-            raw = result["data"]["raw"]
-            # NOTE(hughsaunders): ssrs=scenario specific results
-            ssrs = []
-            for itr in raw:
-                if "output" not in itr:
-                    itr["output"] = {"additive": [], "complete": []}
+                    iteration_output = {"additive": [], "complete": []}
 
                     # NOTE(amaretskiy): "scenario_output" is supported
                     #   for backward compatibility
                     if ("scenario_output" in itr
                             and itr["scenario_output"]["data"]):
-                        itr["output"]["additive"].append(
+                        iteration_output["additive"].append(
                             {"data": itr["scenario_output"]["data"].items(),
                              "title": "Scenario output",
                              "description": "",
                              "chart_plugin": "StackedArea"})
-                        del itr["scenario_output"]
 
-                for idx, additive in enumerate(itr["output"]["additive"]):
-                    try:
-                        for key, value in additive["data"]:
-                            ssrs[idx]["data"][key].append(value)
-                    except IndexError:
-                        data = {}
-                        keys = []
-                        for key, value in additive["data"]:
-                            if key not in data:
-                                data[key] = []
-                                keys.append(key)
-                            data[key].append(value)
-                        ssrs.append({"title": additive["title"],
-                                     "keys": keys,
-                                     "data": data})
-            if not ssrs:
-                return
+                for idx, additive in enumerate(iteration_output["additive"]):
+                    if len(output) <= idx + 1:
+                        output_table = plot.charts.OutputStatsTable(
+                            result["info"], title=additive["title"])
+                        output.append(output_table)
+                    output[idx].add_iteration(additive["data"])
 
-            print("\nScenario Specific Results\n")
+                if itr.get("error"):
+                    task_errors.append(TaskCommands._format_task_error(itr))
 
-            headers = ["key", "min", "median", "90%ile", "95%ile",
-                       "max", "avg"]
-            float_cols = ["min", "median", "90%ile", "95%ile", "max", "avg"]
+            self._print_task_errors(task_id, task_errors)
+
+            cols = plot.charts.MainStatsTable.columns
+            float_cols = cols[1:7]
             formatters = dict(zip(float_cols,
-                              [cliutils.pretty_float_formatter(col, 3)
-                               for col in float_cols]))
+                                  [cliutils.pretty_float_formatter(col, 3)
+                                   for col in float_cols]))
+            rows = [dict(zip(cols, r)) for r in durations.render()["rows"]]
+            cliutils.print_list(rows,
+                                fields=cols,
+                                formatters=formatters,
+                                table_label="Response Times (sec)",
+                                sortby_index=None)
+            print()
 
-            for ssr in ssrs:
-                rows = []
-                for key in ssr["keys"]:
-                    values = ssr["data"][key]
-
-                    if values:
-                        row = [str(key),
-                               round(min(values), 3),
-                               round(utils.median(values), 3),
-                               round(utils.percentile(values, 0.90), 3),
-                               round(utils.percentile(values, 0.95), 3),
-                               round(max(values), 3),
-                               round(utils.mean(values), 3)]
-                    else:
-                        row = [str(key)] + ["n/a"] * 6
-                    rows.append(rutils.Struct(**dict(zip(headers, row))))
-
-                cliutils.print_list(rows,
-                                    fields=headers,
-                                    formatters=formatters,
-                                    table_label=ssr["title"])
+            if iterations_data:
+                formatters = dict(zip(iterations_headers[1:],
+                                      [cliutils.pretty_float_formatter(col, 3)
+                                       for col in iterations_headers[1:]]))
+                cliutils.print_list(iterations,
+                                    fields=iterations_headers,
+                                    table_label="Atomics per iteration",
+                                    formatters=formatters)
                 print()
 
-        def _print_hints(task):
-            print()
-            print("HINTS:")
+            if output:
+                cols = plot.charts.OutputStatsTable.columns
+                float_cols = cols[1:7]
+                formatters = dict(zip(float_cols,
+                                  [cliutils.pretty_float_formatter(col, 3)
+                                   for col in float_cols]))
+
+                for out in output:
+                    data = out.render()
+                    rows = [dict(zip(cols, r)) for r in data["data"]["rows"]]
+                    if rows:
+                        # NOTE(amaretskiy): print title explicitly because
+                        #     prettytable fails if title length is too long
+                        print(data["title"])
+                        cliutils.print_list(rows, fields=cols,
+                                            formatters=formatters)
+                        print()
+
+            print(_("Load duration: %s") %
+                  result["info"]["load_duration"])
+            print(_("Full duration: %s") %
+                  result["info"]["full_duration"])
+
+            print("\nHINTS:")
             print(_("* To plot HTML graphics with this data, run:"))
-            print("\trally task report %s --out output.html" % task["uuid"])
-            print()
+            print("\trally task report %s --out output.html\n" % task["uuid"])
             print(_("* To generate a JUnit report, run:"))
-            print("\trally task report %s --junit --out output.xml" %
+            print("\trally task report %s --junit --out output.xml\n" %
                   task["uuid"])
-            print()
             print(_("* To get raw JSON output of task results, run:"))
             print("\trally task results %s\n" % task["uuid"])
 
-        task = api.Task.get_detailed(task_id)
-
-        if task is None:
-            print("The task %s can not be found" % task_id)
-            return(1)
-
-        if _print_task_info(task):
-            for result in task["results"]:
-                _print_scenario_args(result)
-                _print_summrized_result(result)
-                if iterations_data:
-                    _print_iterations_data(result)
-                _print_ssrs_result(result)
-                print(_("Load duration: %s") %
-                      result["data"]["load_duration"])
-                print(_("Full duration: %s") %
-                      result["data"]["full_duration"])
-            _print_hints(task)
-
     @cliutils.args("--uuid", type=str, dest="task_id", help="UUID of task.")
     @envutils.with_default_task_id
     @cliutils.suppress_warnings
@@ -515,19 +464,21 @@ class TaskCommands(object):
 
         :param task_id: Task uuid
         """
+        task = api.Task.get(task_id)
+        finished_statuses = (consts.TaskStatus.FINISHED,
+                             consts.TaskStatus.ABORTED)
+        if task["status"] not in finished_statuses:
+            print(_("Task status is %s. Results available when it is one "
+                    "of %s.") % (task["status"], ", ".join(finished_statuses)))
+            return 1
+
         results = [{"key": x["key"], "result": x["data"]["raw"],
                     "sla": x["data"]["sla"],
                     "load_duration": x["data"]["load_duration"],
                     "full_duration": x["data"]["full_duration"]}
-                   for x in api.Task.get(task_id).get_results()]
+                   for x in task.get_results()]
 
-        if results:
-            print(json.dumps(results, sort_keys=True, indent=4))
-        else:
-            print(_("The task %s marked as '%s'. Results "
-                    "available when it is '%s' .") % (
-                task_id, consts.TaskStatus.FAILED, consts.TaskStatus.FINISHED))
-            return(1)
+        print(json.dumps(results, sort_keys=True, indent=4))
 
     @cliutils.args("--deployment", dest="deployment", type=str,
                    metavar="<uuid>", required=False,
@@ -701,7 +652,7 @@ class TaskCommands(object):
 
     @cliutils.args("--force", action="store_true", help="force delete")
     @cliutils.args("--uuid", type=str, dest="task_id", nargs="*",
-                   metavar="TASK_ID",
+                   metavar="<task-id>",
                    help="UUID of task or a list of task UUIDs.")
     @envutils.with_default_task_id
     def delete(self, task_id=None, force=False):
@@ -771,3 +722,72 @@ class TaskCommands(object):
         print("Using task: %s" % task_id)
         api.Task.get(task_id)
         fileutils.update_globals_file("RALLY_TASK", task_id)
+
+    @cliutils.args("--uuid", dest="uuid", type=str,
+                   required=True,
+                   help="UUID of a the task.")
+    @cliutils.args("--connection", dest="connection_string", type=str,
+                   required=True,
+                   help="Connection url to the task export system.")
+    @plugins.ensure_plugins_are_loaded
+    def export(self, uuid, connection_string):
+        """Export task results to the custom task's exporting system.
+
+        :param uuid: UUID of the task
+        :param connection_string: string used to connect to the system
+        """
+
+        parsed_obj = urlparse.urlparse(connection_string)
+        try:
+            client = exporter.TaskExporter.get(parsed_obj.scheme)(
+                connection_string)
+        except exceptions.InvalidConnectionString as e:
+            if logging.is_debug():
+                LOG.exception(e)
+            print (e)
+            return 1
+        except exceptions.PluginNotFound as e:
+            if logging.is_debug():
+                LOG.exception(e)
+            msg = ("\nPlease check your connection string. The format of "
+                   "`connection` should be plugin-name://"
+                   "<user>:<pwd>@<full_address>:<port>/<path>.<type>")
+            print (str(e) + msg)
+            return 1
+
+        try:
+            client.export(uuid)
+        except (IOError, exceptions.RallyException) as e:
+            if logging.is_debug():
+                LOG.exception(e)
+            print (e)
+            return 1
+        print(_("Task %(uuid)s results was successfully exported to %("
+                "connection)s using %(name)s plugin.") % {
+                    "uuid": uuid,
+                    "connection": connection_string,
+                    "name": parsed_obj.scheme
+        })
+
+    @staticmethod
+    def _print_task_errors(task_id, task_errors):
+        print(cliutils.make_header("Task %s has %d error(s)" %
+                                   (task_id, len(task_errors))))
+        for err_data in task_errors:
+            print(*err_data, sep="\n")
+            print("-" * 80)
+
+    @staticmethod
+    def _format_task_error(data):
+        error_type = _("Unknown type")
+        error_message = _("Rally hasn't caught anything yet")
+        error_traceback = _("No traceback available.")
+        try:
+            error_type = data["error"][0]
+            error_message = data["error"][1]
+            error_traceback = data["error"][2]
+        except IndexError:
+            pass
+        return ("%(error_type)s: %(error_message)s\n" %
+                {"error_type": error_type, "error_message": error_message},
+                error_traceback)
