@@ -16,12 +16,86 @@
 import json
 import uuid
 
+from rally.common import costilius
 from rally.common import db
 from rally.common.i18n import _LE
 from rally import consts
 from rally import exceptions
 
 
+OUTPUT_SCHEMA = {
+    "type": "object",
+    "properties": {
+        "additive": {
+            "type": "array",
+            "items": {
+                "type": "object",
+                "properties": {
+                    "title": {"type": "string"},
+                    "description": {"type": "string"},
+                    "chart_plugin": {"type": "string"},
+                    "data": {
+                        "type": "array",
+                        "items": {
+                            "type": "array",
+                            "items": [{"type": "string"},
+                                      {"type": "number"}],
+                            "additionalItems": False}},
+                    "label": {"type": "string"},
+                    "axis_label": {"type": "string"}},
+                "required": ["title", "chart_plugin", "data"],
+                "additionalProperties": False
+            }
+        },
+        "complete": {
+            "type": "array",
+            "items": {
+                "type": "object",
+                "properties": {
+                    "title": {"type": "string"},
+                    "description": {"type": "string"},
+                    "chart_plugin": {"type": "string"},
+                    "data": {"anyOf": [
+                        {"type": "array",
+                         "items": {
+                             "type": "array",
+                             "items": [
+                                 {"type": "string"},
+                                 {"anyOf": [
+                                     {"type": "array",
+                                      "items": {"type": "array",
+                                                "items": [{"type": "number"},
+                                                          {"type": "number"}]
+                                                }},
+                                     {"type": "number"}]}]}},
+                        {"type": "object",
+                         "properties": {
+                             "cols": {"type": "array",
+                                      "items": {"type": "string"}},
+                             "rows": {
+                                 "type": "array",
+                                 "items": {
+                                     "type": "array",
+                                     "items": {"anyOf": [{"type": "string"},
+                                                         {"type": "number"}]}}
+                             }
+                         },
+                         "required": ["cols", "rows"],
+                         "additionalProperties": False}
+                    ]},
+                    "label": {"type": "string"},
+                    "axis_label": {"type": "string"}
+                },
+                "required": ["title", "chart_plugin", "data"],
+                "additionalProperties": False
+            }
+        }
+    },
+    "required": ["additive", "complete"],
+    "additionalProperties": False
+}
+
+
 TASK_RESULT_SCHEMA = {
     "type": "object",
     "$schema": consts.JSON_SCHEMA,
@@ -75,6 +149,8 @@ TASK_RESULT_SCHEMA = {
                     "idle_duration": {
                         "type": "number"
                     },
+                    # NOTE(amaretskiy): "scenario_output" is deprecated
+                    #                   in favor of "output"
                     "scenario_output": {
                         "type": "object",
                         "properties": {
@@ -87,9 +163,10 @@ TASK_RESULT_SCHEMA = {
                         },
                         "required": ["data", "errors"]
                     },
+                    "output": OUTPUT_SCHEMA
                 },
                 "required": ["atomic_actions", "duration", "error",
-                             "idle_duration", "scenario_output"]
+                             "idle_duration"]
             },
             "minItems": 1
         },
@@ -162,17 +239,10 @@ TASK_EXTENDED_RESULT_SCHEMA = {
                     "idle_duration": {
                         "type": "number"
                     },
-                    "scenario_output": {
-                        "type": "object",
-                        "properties": {
-                            "data": {"type": "object"},
-                            "errors": {"type": "string"},
-                        },
-                        "required": ["data", "errors"]
-                    },
+                    "output": OUTPUT_SCHEMA
                 },
                 "required": ["atomic_actions", "duration", "error",
-                             "idle_duration", "scenario_output"]
+                             "idle_duration", "output"]
             },
             "minItems": 1
         },
@@ -192,7 +262,6 @@ TASK_EXTENDED_RESULT_SCHEMA = {
             "type": "object",
             "properties": {
                 "atomic": {"type": "object"},
-                "output_names": {"type": "array"},
                 "iterations_count": {"type": "integer"},
                 "iterations_failed": {"type": "integer"},
                 "min_duration": {"type": "number"},
@@ -218,17 +287,18 @@ class Task(object):
     NOT_IMPLEMENTED_STAGES_FOR_ABORT = [consts.TaskStatus.VERIFYING,
                                         consts.TaskStatus.INIT]
 
-    def __init__(self, task=None, fake=False, **attributes):
+    def __init__(self, task=None, temporary=False, **attributes):
         """Task object init
 
         :param task: dictionary like object, that represents a task
-        :param fake: if True, will be created task object with random UUID and
-            parameters, passed in 'attributes'. Does not create database
-            record. Used for special purposes, like task config validation.
+        :param temporary: whenever this param is True the task will be created
+            with a random UUID and no database record. Used for special
+            purposes, like task config validation.
         """
 
-        self.fake = fake
-        if fake:
+        self.is_temporary = temporary
+
+        if self.is_temporary:
             self.task = task or {"uuid": str(uuid.uuid4())}
             self.task.update(attributes)
         else:
@@ -243,6 +313,10 @@ class Task(object):
         db_task["deployment_name"] = deployment_name
         return db_task
 
+    @staticmethod
+    def get_detailed(task_id):
+        return db.api.task_get_detailed(task_id)
+
     @staticmethod
     def get(uuid):
         return Task(db.task_get(uuid))
@@ -260,7 +334,7 @@ class Task(object):
         db.task_delete(uuid, status=status)
 
     def _update(self, values):
-        if not self.fake:
+        if not self.is_temporary:
             self.task = db.task_update(self.task["uuid"], values)
 
     def update_status(self, status, allowed_statuses=None):
@@ -272,9 +346,12 @@ class Task(object):
     def update_verification_log(self, log):
         self._update({"verification_log": json.dumps(log)})
 
-    def set_failed(self, log=""):
+    def set_failed(self, etype, msg, etraceback):
         self._update({"status": consts.TaskStatus.FAILED,
-                      "verification_log": json.dumps(log)})
+                      "verification_log": json.dumps([etype,
+                                                      msg,
+                                                      etraceback
+                                                      ])})
 
     def get_results(self):
         return db.task_result_get_all_by_uuid(self.task["uuid"])
@@ -299,17 +376,16 @@ class Task(object):
         :returns: list of dicts, each dict represents scenario results:
                   key - dict, scenario input data
                   sla - list, SLA results
-                  iterations - if serializiable, then iterator with
+                  iterations - if serializable, then iterator with
                                iterations data, otherwise a list
-                  created_at - if serializiable, then str datetime,
+                  created_at - if serializable, then str datetime,
                                otherwise absent
-                  updated_at - if serializiable, then str datetime,
+                  updated_at - if serializable, then str datetime,
                                otherwise absent
                   info:
                       atomic - dict where key is one of atomic action names
                                and value is dict {min_duration: number,
                                                   max_duration: number}
-                      output_names - list of str output values names (if any)
                       iterations_count - int number of iterations
                       iterations_failed - int number of iterations with errors
                       min_duration - float minimum iteration duration
@@ -325,8 +401,7 @@ class Task(object):
             min_duration = 0
             max_duration = 0
             iterations_failed = 0
-            atomic = {}
-            output_names = set()
+            atomic = costilius.OrderedDict()
 
             for itr in scenario["data"]["raw"]:
                 for atomic_name, duration in itr["atomic_actions"].items():
@@ -339,11 +414,23 @@ class Task(object):
                     elif duration > atomic[atomic_name]["max_duration"]:
                         atomic[atomic_name]["max_duration"] = duration
 
-                output_names.update(itr["scenario_output"]["data"].keys())
-
                 if not tstamp_start or itr["timestamp"] < tstamp_start:
                     tstamp_start = itr["timestamp"]
 
+                if "output" not in itr:
+                    itr["output"] = {"additive": [], "complete": []}
+
+                    # NOTE(amaretskiy): Deprecated "scenario_output"
+                    #     is supported for backward compatibility
+                    if ("scenario_output" in itr
+                            and itr["scenario_output"]["data"]):
+                        itr["output"]["additive"].append(
+                            {"items": itr["scenario_output"]["data"].items(),
+                             "title": "Scenario output",
+                             "description": "",
+                             "chart": "OutputStackedAreaChart"})
+                        del itr["scenario_output"]
+
                 if itr["error"]:
                     iterations_failed += 1
                 else:
@@ -364,7 +451,6 @@ class Task(object):
 
             scenario["info"] = {
                 "atomic": atomic,
-                "output_names": list(output_names),
                 "iterations_count": len(scenario["data"]["raw"]),
                 "iterations_failed": iterations_failed,
                 "min_duration": min_duration,
