@@ -13,24 +13,9 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-
-import sys
-
-from oslo_config import cfg
+import six
 
 from rally.common.i18n import _
-from rally.common import log as logging
-
-LOG = logging.getLogger(__name__)
-
-EXC_LOG_OPTS = [
-    cfg.BoolOpt("fatal_exception_format_errors",
-                default=False,
-                help="make exception message format errors fatal"),
-]
-
-CONF = cfg.CONF
-CONF.register_opts(EXC_LOG_OPTS)
 
 
 class RallyException(Exception):
@@ -46,37 +31,13 @@ class RallyException(Exception):
     def __init__(self, message=None, **kwargs):
         self.kwargs = kwargs
 
-        if "code" not in self.kwargs:
-            try:
-                self.kwargs["code"] = self.code
-            except AttributeError:
-                pass
-
         if "%(message)s" in self.msg_fmt:
             kwargs.update({"message": message})
 
-        try:
-            message = self.msg_fmt % kwargs
-        except KeyError:
-            exc_info = sys.exc_info()
-            # kwargs doesn't match a variable in the message
-            # log the issue and the kwargs
-            msg = "kwargs don't match in string format operation: %s"
-            LOG.debug(msg % kwargs, exc_info=exc_info)
-
-            if CONF.fatal_exception_format_errors:
-                raise exc_info[0].with_traceback(exc_info[2])
-            else:
-                # at least get the core message out if something happened
-                message = message or self.msg_fmt
-
-        super(RallyException, self).__init__(message)
+        super(RallyException, self).__init__(self.msg_fmt % kwargs)
 
     def format_message(self):
-        if self.__class__.__name__.endswith("_Remote"):
-            return self.args[0]
-        else:
-            return unicode(self)
+        return six.text_type(self)
 
 
 class ImmutableException(RallyException):
@@ -97,50 +58,38 @@ class InvalidRunnerResult(RallyException):
 
 
 class InvalidTaskException(InvalidConfigException):
-    msg_fmt = _("This config is invalid: `%(message)s`")
+    msg_fmt = _("Task config is invalid: `%(message)s`")
 
 
 class NotFoundScenarios(InvalidTaskException):
     msg_fmt = _("There are no benchmark scenarios with names: `%(names)s`.")
 
 
-class InvalidBenchmarkConfig(InvalidTaskException):
+class InvalidTaskConfig(InvalidTaskException):
     msg_fmt = _("Input task is invalid!\n\n"
-                "Benchmark %(name)s[%(pos)s] has wrong configuration"
-                "\nBenchmark configuration:\n%(config)s\n"
+                "Subtask %(name)s[%(pos)s] has wrong configuration"
+                "\Subtask configuration:\n%(config)s\n"
                 "\nReason:\n %(reason)s")
 
 
 class NotFoundException(RallyException):
-    msg_fmt = _("Not found.")
+    msg_fmt = _("The resource can not be found: %(message)s")
 
 
-class NoSuchPlugin(NotFoundException):
-    msg_fmt = _("There is no plugin with name: `%(name)s`.")
+class ThreadTimeoutException(RallyException):
+    msg_fmt = _("Iteration interrupted due to timeout.")
 
 
-class NoSuchEngine(NotFoundException):
-    msg_fmt = _("There is no engine with name `%(engine_name)s`.")
+class PluginNotFound(NotFoundException):
+    msg_fmt = _("There is no plugin with name: `%(name)s` in "
+                "%(namespace)s namespace.")
 
 
-class NoSuchVMProvider(NotFoundException):
-    msg_fmt = _("There is no vm provider with name `%(vm_provider_name)s`.")
-
-
-class NoSuchScenario(NotFoundException):
-    msg_fmt = _("There is no benchmark scenario with name `%(name)s`.")
-
-
-class NoSuchRunner(NotFoundException):
-    msg_fmt = _("There is no benchmark runner with type `%(type)s`.")
-
-
-class NoSuchContext(NotFoundException):
-    msg_fmt = _("There is no benchmark context with name `%(name)s`.")
-
-
-class NoSuchSLA(NotFoundException):
-    msg_fmt = _("There is no SLA with name `%(name)s`.")
+class PluginWithSuchNameExists(RallyException):
+    msg_fmt = _("Plugin with such name: %(name)s already exists in "
+                "%(namespace)s namespace. It's module allocates at "
+                "%(existing_path)s. You are trying to add plugin whose module "
+                "allocates at %(new_path)s.")
 
 
 class NoSuchConfigField(NotFoundException):
@@ -191,7 +140,8 @@ class GetResourceNotFound(GetResourceFailure):
 
 
 class GetResourceErrorStatus(GetResourceFailure):
-    msg_fmt = _("Resource %(resource)s has %(status)s status: %(fault)s")
+    msg_fmt = _("Resource %(resource)s has %(status)s status.\n"
+                "Fault: %(fault)s")
 
 
 class ScriptError(RallyException):
@@ -284,3 +234,20 @@ class InvalidHostException(RallyException):
 
 class MultipleMatchesFound(RallyException):
     msg_fmt = _("Found multiple %(needle)s: %(haystack)s")
+
+
+class TempestConfigCreationFailure(RallyException):
+    msg_fmt = _("Unable to create Tempest config file: %(message)s")
+
+
+class SSHTimeout(RallyException):
+    pass
+
+
+class SSHError(RallyException):
+    pass
+
+
+class InvalidConnectionString(RallyException):
+    msg_fmt = _("The connection string is not valid: %(message)s. Please "
+                "check your connection string.")
\ No newline at end of file
