@@ -24,6 +24,7 @@ from rally.plugins.openstack.cleanup import base
 from rally.plugins.openstack.scenarios.fuel import utils as futils
 from rally.plugins.openstack.scenarios.keystone import utils as kutils
 from rally.plugins.openstack.scenarios.nova import utils as nova_utils
+from rally.plugins.openstack.wrappers import glance as glance_wrapper
 from rally.plugins.openstack.wrappers import keystone as keystone_wrapper
 
 LOG = logging.getLogger(__name__)
@@ -58,7 +59,8 @@ class QuotaMixin(SynchronizedDeletion):
 
 @base.resource("heat", "stacks", order=100, tenant_resource=True)
 class HeatStack(base.ResourceManager):
-    pass
+    def name(self):
+        return self.raw_resource.stack_name
 
 
 # NOVA
@@ -186,8 +188,8 @@ class EC2Server(EC2Mixin, base.ResourceManager):
 
         # NOTE(wtakase): After instance deletion, instance can be 'terminated'
         #                state. If all instance states are 'terminated', this
-        #                returns True. And if get_only_instaces() returns empty
-        #                list, this also returns True because we consider
+        #                returns True. And if get_only_instances() returns an
+        #                empty list, this also returns True because we consider
         #                instance has already been deleted.
         return all(map(lambda i: i.state == "terminated", instances))
 
@@ -221,7 +223,7 @@ class NeutronMixin(SynchronizedDeletion, base.ResourceManager):
         return self.raw_resource["id"]
 
     def name(self):
-        return self.raw_resource["name"]
+        return self.raw_resource.get("name", "")
 
     def delete(self):
         delete_method = getattr(self._manager(), "delete_%s" % self._resource)
@@ -309,7 +311,12 @@ class NeutronFloatingIP(NeutronMixin):
 @base.resource("neutron", "security_group", order=next(_neutron_order),
                tenant_resource=True)
 class NeutronSecurityGroup(NeutronMixin):
-    pass
+    def list(self):
+        tenant_sgs = super(NeutronSecurityGroup, self).list()
+        # NOTE(pirsriva): Filter out "default" security group deletion
+        # by non-admin role user
+        return filter(lambda r: r["name"] != "default",
+                      tenant_sgs)
 
 
 @base.resource("neutron", "quota", order=next(_neutron_order),
@@ -383,8 +390,15 @@ class ManilaSecurityService(base.ResourceManager):
 @base.resource("glance", "images", order=500, tenant_resource=True)
 class GlanceImage(base.ResourceManager):
 
+    def _wrapper(self):
+        return glance_wrapper.wrap(
+            getattr(self.admin or self.user, self._service), self)
+
     def list(self):
-        return self._manager().list(owner=self.tenant_uuid)
+        return self._wrapper().list_images(owner=self.tenant_uuid)
+
+    def delete(self):
+        return self._wrapper().delete_image(self.raw_resource)
 
 
 # SAHARA
