@@ -33,7 +33,7 @@ class CinderVolumes(cinder_utils.CinderScenario,
                     glance_utils.GlanceScenario):
     """Benchmark scenarios for Cinder Volumes."""
 
-    @types.set(image=types.ImageResourceType)
+    @types.convert(image={"type": "glance_image"})
     @validation.image_exists("image", nullable=True)
     @validation.required_services(consts.Service.CINDER)
     @validation.required_openstack(users=True)
@@ -80,7 +80,7 @@ class CinderVolumes(cinder_utils.CinderScenario,
 
         self._list_volumes(detailed)
 
-    @types.set(image=types.ImageResourceType)
+    @types.convert(image={"type": "glance_image"})
     @validation.image_exists("image", nullable=True)
     @validation.required_services(consts.Service.CINDER)
     @validation.required_openstack(users=True)
@@ -102,7 +102,7 @@ class CinderVolumes(cinder_utils.CinderScenario,
         volume = self._create_volume(size, **create_volume_kwargs)
         self._update_volume(volume, **update_volume_kwargs)
 
-    @types.set(image=types.ImageResourceType)
+    @types.convert(image={"type": "glance_image"})
     @validation.image_exists("image", nullable=True)
     @validation.required_services(consts.Service.CINDER)
     @validation.required_openstack(users=True)
@@ -135,7 +135,7 @@ class CinderVolumes(cinder_utils.CinderScenario,
         self.sleep_between(min_sleep, max_sleep)
         self._delete_volume(volume)
 
-    @types.set(image=types.ImageResourceType)
+    @types.convert(image={"type": "glance_image"})
     @validation.image_exists("image", nullable=True)
     @validation.required_services(consts.Service.CINDER)
     @validation.required_openstack(users=True)
@@ -272,13 +272,18 @@ class CinderVolumes(cinder_utils.CinderScenario,
         self.sleep_between(min_sleep, max_sleep)
         self._delete_snapshot(snapshot)
 
-    @types.set(image=types.ImageResourceType,
-               flavor=types.FlavorResourceType)
+    @types.convert(image={"type": "glance_image"},
+                   flavor={"type": "nova_flavor"})
     @validation.image_valid_on_flavor("flavor", "image")
     @validation.required_services(consts.Service.NOVA, consts.Service.CINDER)
     @validation.required_openstack(users=True)
     @scenario.configure(context={"cleanup": ["cinder", "nova"]})
-    def create_and_attach_volume(self, size, image, flavor, **kwargs):
+    @logging.log_deprecated_args(
+        "Use 'create_vm_params' for additional instance parameters.",
+        "0.2.0", ["kwargs"], once=True)
+    def create_and_attach_volume(self, size, image, flavor,
+                                 create_volume_params=None,
+                                 create_vm_params=None, **kwargs):
         """Create a VM and attach a volume to it.
 
         Simple test to create a VM and attach a volume, then
@@ -290,11 +295,22 @@ class CinderVolumes(cinder_utils.CinderScenario,
                          max - maximum size volumes will be created as.
         :param image: Glance image name to use for the VM
         :param flavor: VM flavor name
-        :param kwargs: optional arguments for VM creation
+        :param create_volume_params: optional arguments for volume creation
+        :param create_vm_params: optional arguments for VM creation
+        :param kwargs: (deprecated) optional arguments for VM creation
         """
 
-        server = self._boot_server(image, flavor, **kwargs)
-        volume = self._create_volume(size)
+        create_volume_params = create_volume_params or {}
+
+        if kwargs and create_vm_params:
+            raise ValueError("You can not set both 'kwargs'"
+                             "and 'create_vm_params' attributes."
+                             "Please use 'create_vm_params'.")
+
+        create_vm_params = create_vm_params or kwargs or {}
+
+        server = self._boot_server(image, flavor, **create_vm_params)
+        volume = self._create_volume(size, **create_volume_params)
 
         self._attach_volume(server, volume)
         self._detach_volume(server, volume)
@@ -348,11 +364,9 @@ class CinderVolumes(cinder_utils.CinderScenario,
     @validation.required_services(consts.Service.NOVA, consts.Service.CINDER)
     @validation.required_openstack(users=True)
     @scenario.configure(context={"cleanup": ["cinder", "nova"]})
-    @logging.log_deprecated_args("Use 'nested_level' as an int", "0.1.2",
-                                 ["nested_level"], once=True)
     def create_nested_snapshots_and_attach_volume(self,
                                                   size=None,
-                                                  nested_level=None,
+                                                  nested_level=1,
                                                   **kwargs):
 
         """Create a volume from snapshot and attach/detach the volume
@@ -366,25 +380,12 @@ class CinderVolumes(cinder_utils.CinderScenario,
                         min - minimum size volumes will be created as;
                         max - maximum size volumes will be created as.
                      default values: {"min": 1, "max": 5}
-        :param nested_level: Nested level - dictionary or int, dictionary
-                             contains two values:
-                               min - minimum number of volumes will be created
-                                     from snapshot;
-                               max - maximum number of volumes will be created
-                                     from snapshot.
-                             due to its deprecated would be taken min value.
-                             int, means the exact nested level.
-                             default value: 1.
+        :param nested_level: amount of nested levels
         :param kwargs: Optional parameters used during volume
                        snapshot creation.
         """
         if size is None:
             size = {"min": 1, "max": 5}
-        if nested_level is None:
-            nested_level = 1
-        nested_level = nested_level or 1
-        if isinstance(nested_level, dict):
-            nested_level = nested_level.get("min", 1)
 
         # NOTE: Volume size cannot be smaller than the snapshot size, so
         #       volume with specified size should be created to avoid
