@@ -60,13 +60,12 @@ class GlanceWrapper(object):
         self.owner = owner
         self.client = client
 
-    @abc.abstractmethod
-    def get_image(self, image):
-        """Refresh an image from an image object."""
-
     @abc.abstractmethod
     def create_image(self, container_format, image_location, disk_format):
-        """Creates new image."""
+        """Creates new image.
+
+        Accepts all Glance v2 parameters.
+        """
 
     @abc.abstractmethod
     def delete_image(self, image):
@@ -74,13 +73,13 @@ class GlanceWrapper(object):
 
     @abc.abstractmethod
     def list_images(self, **filters):
-        """List images."""
+        """List images.
+
+        Accepts all Glance v2 filters.
+        """
 
 
 class GlanceV1Wrapper(GlanceWrapper):
-    def get_image(self, image):
-        return image.manager.get(image.id)
-
     def create_image(self, container_format, image_location,
                      disk_format, **kwargs):
         kw = {
@@ -90,6 +89,9 @@ class GlanceV1Wrapper(GlanceWrapper):
         kw.update(kwargs)
         if "name" not in kw:
             kw["name"] = self.owner.generate_random_name()
+        if "visibility" in kw:
+            kw["is_public"] = kw.pop("visibility") == "public"
+
         image_location = os.path.expanduser(image_location)
 
         try:
@@ -124,16 +126,25 @@ class GlanceV1Wrapper(GlanceWrapper):
             check_interval=CONF.benchmark.glance_image_delete_poll_interval)
 
     def list_images(self, **filters):
-        return self.client.images.list(**filters)
+        kwargs = {"filters": filters}
+        if "owner" in filters:
+            # NOTE(stpierre): in glance v1, "owner" is not a filter,
+            # so we need to handle it separately.
+            kwargs["owner"] = kwargs["filters"].pop("owner")
+        visibility = kwargs["filters"].pop("visibility", None)
+        images = self.client.images.list(**kwargs)
+        # NOTE(stpierre): Glance v1 isn't smart enough to filter on
+        # public/private images, so we have to do it manually.
+        if visibility is not None:
+            is_public = visibility == "public"
+            return [i for i in images if i.is_public is is_public]
+        return images
 
 
 class GlanceV2Wrapper(GlanceWrapper):
-    def get_image(self, image):
-        return self.client.images.get(image.id)
-
     def _update_image(self, image):
         try:
-            return self.get_image(image)
+            return self.client.images.get(image.id)
         except glance_exc.HTTPNotFound:
             raise exceptions.GetResourceNotFound(resource=image)
 
@@ -163,16 +174,19 @@ class GlanceV2Wrapper(GlanceWrapper):
         timeout = time.time() - start
 
         image_data = None
+        response = None
         try:
             if os.path.isfile(image_location):
                 image_data = open(image_location)
             else:
-                response = requests.get(image_location)
+                response = requests.get(image_location, stream=True)
                 image_data = response.raw
             self.client.images.upload(image.id, image_data)
         finally:
             if image_data is not None:
                 image_data.close()
+            if response is not None:
+                response.close()
 
         return utils.wait_for_status(
             image, ["active"],
