@@ -14,11 +14,11 @@
 
 import abc
 import bisect
+import collections
 import math
 
 import six
 
-from rally.common import costilius
 from rally.common.plugin import plugin
 from rally.common import streaming_algorithms as streaming
 from rally.task.processing import utils
@@ -27,7 +27,12 @@ from rally.task.processing import utils
 @six.add_metaclass(abc.ABCMeta)
 @plugin.configure(name="base_output_chart")
 class Chart(plugin.Plugin):
-    """Base class for charts."""
+    """Base class for charts.
+
+    This is a base for all plugins that prepare data for specific charts
+    in HTML report. Each chart must at least declare chart widget and
+    prepare data that is suitable for rendering by JavaScript.
+    """
 
     @abc.abstractproperty
     def widget(self):
@@ -41,7 +46,7 @@ class Chart(plugin.Plugin):
                                that should have int value of total data size
         :param zipped_size: int maximum number of points on scale
         """
-        self._data = costilius.OrderedDict()  # Container for results
+        self._data = collections.OrderedDict()  # Container for results
         self._workload_info = workload_info
         self.base_size = workload_info.get("iterations_count", 0)
         self.zipped_size = zipped_size
@@ -146,7 +151,7 @@ class LoadProfileChart(Chart):
     widget = "StackedArea"
 
     def __init__(self, workload_info, name="parallel iterations",
-                 scale=200):
+                 scale=100):
         """Setup chart with graph name and scale.
 
         :workload_info:  dict, generalized info about iterations
@@ -155,49 +160,41 @@ class LoadProfileChart(Chart):
         """
         super(LoadProfileChart, self).__init__(workload_info)
         self._name = name
-        self._duration = workload_info["load_duration"]
+        # NOTE(boris-42): Add 2 points at the end of graph so at the end of
+        #                 graph there will be point with 0 running iterations.
+        self._duration = workload_info["load_duration"] * (1 + 2.0 / scale)
         self._tstamp_start = workload_info["tstamp_start"]
 
-        # NOTE(amaretskiy): Determine a chart `step' - duration between
-        #   two X points, rounded with minimal accuracy (digits after point)
-        #   to improve JavaScript drawing performance.
-        # Examples:
-        #   scale  duration       step (initial)    accuracy  step
-        #   200    30.8043010235  0.154021505117    1         0.2
-        #   200    1.25884699821  0.00629423499107  3         0.006
-        step = self._duration / float(scale)
-        if step == 0:
-            accuracy = 0
-        else:
-            accuracy = max(-int(math.floor(math.log10(step))), 0)
-        step = round(step, accuracy)
-        self._time_axis = [step * x
-                           for x in six.moves.range(1, int(scale))
-                           if (step * x) < self._duration]
+        self.step = self._duration / float(scale)
+        self._time_axis = [self.step * x
+                           for x in six.moves.range(int(scale))
+                           if (self.step * x) < self._duration]
         self._time_axis.append(self._duration)
-        self._started = [0] * len(self._time_axis)
-        self._stopped = [0] * len(self._time_axis)
+        self._running = [0] * len(self._time_axis)
 
     def _map_iteration_values(self, iteration):
-        return (iteration["timestamp"],
-                0 if iteration["error"] else iteration["duration"])
+        return (iteration["timestamp"], iteration["duration"])
 
     def add_iteration(self, iteration):
         timestamp, duration = self._map_iteration_values(iteration)
         ts_start = timestamp - self._tstamp_start
-        ts_stop = ts_start + duration
-        self._started[bisect.bisect(self._time_axis, ts_start)] += 1
-        self._stopped[bisect.bisect(self._time_axis, ts_stop)] += 1
+        started_idx = bisect.bisect(self._time_axis, ts_start)
+        ended_idx = bisect.bisect(self._time_axis, ts_start + duration)
+        if self._time_axis[ended_idx - 1] == ts_start + duration:
+            ended_idx -= 1
+        for idx in range(started_idx + 1, ended_idx):
+            self._running[idx] += 1
+        if started_idx == ended_idx:
+            self._running[ended_idx] += duration / self.step
+        else:
+            self._running[started_idx] += (
+                self._time_axis[started_idx] - ts_start) / self.step
+            self._running[ended_idx] += (
+                ts_start + duration
+                - self._time_axis[ended_idx - 1]) / self.step
 
     def render(self):
-        data = []
-        running = 0
-        for ts, started, ended in zip(self._time_axis,
-                                      self._started, self._stopped):
-            running += started
-            data.append([ts, running])
-            running -= ended
-        return [(self._name, data)]
+        return [(self._name, list(zip(self._time_axis, self._running)))]
 
 
 class HistogramChart(Chart):
@@ -325,8 +322,12 @@ class Table(Chart):
         :returns: bool
         """
         for ins, fn in values:
-            if isinstance(ins, streaming.MinComputation):
-                return bool(ins.result())
+            if isinstance(ins, (streaming.MinComputation,
+                                streaming.MaxComputation,
+                                streaming.MeanComputation)):
+                # NOTE(amaretskiy): None means this computation
+                #                   has never been called
+                return ins.result() is not None
         return True
 
     def get_rows(self):
@@ -570,3 +571,51 @@ class OutputStatsTable(OutputTable):
             self._data[name][-2][0].add(1)
             for idx, dummy in enumerate(self._data[name][:-1]):
                 self._data[name][idx][0].add(value)
+
+
+_OUTPUT_SCHEMA = {
+    "key_types": {
+        "title": str, "description": str, "chart_plugin": str,
+        "data": (list, dict), "label": str, "axis_label": str},
+    "required": ["title", "chart_plugin", "data"]}
+
+
+def validate_output(output_type, output):
+    # TODO(amaretskiy): this validation is simple and must be improved.
+    #   Maybe it is worth to add classmethod OutputChart.validate(), so
+    #   we could have flexible validation for custom chart plugins
+    if output_type not in ("additive", "complete"):
+        return ("unexpected output type: '%s', "
+                "should be in ('additive', 'complete')" % output_type)
+
+    if type(output) != dict:
+        return ("%(name)s output item has wrong type '%(type)s', "
+                "must be 'dict'" % {"name": output_type,
+                                    "type": type(output).__name__})
+
+    for key in _OUTPUT_SCHEMA["required"]:
+        if key not in output:
+            return ("%(name)s output missing key '%(key)s'"
+                    % {"name": output_type, "key": key})
+
+    for key in output:
+        if key not in _OUTPUT_SCHEMA["key_types"]:
+            return ("%(name)s output has unexpected key '%(key)s'"
+                    % {"name": output_type, "key": key})
+
+        proper_type = _OUTPUT_SCHEMA["key_types"][key]
+        if not isinstance(output[key], proper_type):
+            if type(proper_type) == tuple:
+                return ("Value of %(name)s output %(key)s has wrong type "
+                        "'%(actual_type)s', should be in %(types)r"
+                        % {"name": output_type,
+                           "key": key,
+                           "actual_type": type(output[key]).__name__,
+                           "types": tuple(t.__name__
+                                          for t in proper_type)})
+            return ("Value of %(name)s output %(key)s has wrong type "
+                    "'%(actual_type)s', should be %(proper_type)s"
+                    % {"name": output_type,
+                       "key": key,
+                       "actual_type": type(output[key]).__name__,
+                       "proper_type": proper_type.__name__})
