@@ -15,17 +15,18 @@
 
 
 import random
-import time
 
 import six
 
 from rally.common import logging
+from rally.common.objects import task  # noqa
 from rally.common.plugin import plugin
 from rally.common import utils
 from rally import consts
 from rally import exceptions
 from rally.task import atomic
 from rally.task import functional
+from rally.task.processing import charts
 
 
 LOG = logging.getLogger(__name__)
@@ -106,7 +107,8 @@ class Scenario(plugin.Plugin,
         super(Scenario, self).__init__()
         self.context = context or {}
         self.task = self.context.get("task", {})
-        self._idle_duration = 0
+        self._idle_duration = 0.0
+        self._output = {"additive": [], "complete": []}
 
     @staticmethod
     def _validate_helper(validators, clients, config, deployment):
@@ -142,8 +144,8 @@ class Scenario(plugin.Plugin,
             for user in users:
                 cls._validate_helper(user_validators, user, config, deployment)
 
-    def sleep_between(self, min_sleep, max_sleep):
-        """Performs a time.sleep() call for a random amount of seconds.
+    def sleep_between(self, min_sleep, max_sleep, atomic_delay=0.1):
+        """Call an interruptable_sleep() for a random amount of seconds.
 
         The exact time is chosen uniformly randomly from the interval
         [min_sleep; max_sleep). The method also updates the idle_duration
@@ -151,15 +153,31 @@ class Scenario(plugin.Plugin,
 
         :param min_sleep: Minimum sleep time in seconds (non-negative)
         :param max_sleep: Maximum sleep time in seconds (non-negative)
+        :param atomic_delay: parameter with which  time.sleep would be called
+                             int(sleep_time / atomic_delay) times.
         """
         if not 0 <= min_sleep <= max_sleep:
             raise exceptions.InvalidArgumentsException(
                 "0 <= min_sleep <= max_sleep")
 
         sleep_time = random.uniform(min_sleep, max_sleep)
-        time.sleep(sleep_time)
+        utils.interruptable_sleep(sleep_time, atomic_delay)
         self._idle_duration += sleep_time
 
     def idle_duration(self):
         """Returns duration of all sleep_between."""
         return self._idle_duration
+
+    def add_output(self, additive=None, complete=None):
+        """Add iteration values for additive output.
+
+        :param additive: dict with additive output
+        :param complete: dict with complete output
+        :raises RallyException: When additive or complete has wrong format
+        """
+        for key, value in (("additive", additive), ("complete", complete)):
+            if value:
+                message = charts.validate_output(key, value)
+                if message:
+                    raise exceptions.RallyException(message)
+                self._output[key].append(value)
