@@ -620,7 +620,7 @@ def volume_type_exists(config, clients, deployment, param_name):
     val = config.get("args", {}).get(param_name)
     if val:
         volume_types_list = clients.cinder().volume_types.list()
-        if len(volume_types_list) < 1:
+        if not volume_types_list:
             message = (_("Must have at least one volume type created "
                          "when specifying use of volume types."))
             return ValidationResult(False, message)
@@ -649,3 +649,37 @@ def restricted_parameters(config, clients, deployment, param_names,
                % {"params": ", ".join(restricted_params),
                   "a_dict": subdict if subdict else "args"})
         return ValidationResult(False, msg)
+
+
+@validator
+def validate_heat_template(config, clients, deployment, *param_names):
+    """Validates heat template.
+
+    :param param_names: list of parameters to be validated.
+    """
+    if param_names is None:
+        return ValidationResult(False, _(
+            "validate_heat_template validator accepts non empty arguments "
+            "in form of `validate_heat_template(\"foo\", \"bar\")`"))
+    for param_name in param_names:
+        template_path = config.get("args", {}).get(param_name)
+        if not template_path:
+            return ValidationResult(False, _(
+                "Path to heat template is not specified. Its needed for "
+                "heat template validation. Please check the content of `%s` "
+                "scenario argument.") % param_name)
+        template_path = os.path.expanduser(template_path)
+        if not os.path.exists(template_path):
+            return ValidationResult(False, _("No file found by the given path "
+                                             "%s") % template_path)
+        with open(template_path, "r") as f:
+            try:
+                clients.heat().stacks.validate(template=f.read())
+            except Exception as e:
+                dct = {
+                    "path": template_path,
+                    "msg": str(e),
+                }
+                msg = (_("Heat template validation failed on %(path)s. "
+                         "Original error message: %(msg)s.") % dct)
+                return ValidationResult(False, msg)
