@@ -13,10 +13,12 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from distutils import version
 import inspect
 import os
 import re
 
+from neutronclient import version as nc_version
 from oslo_config import cfg
 import requests
 import six
@@ -48,14 +50,14 @@ IMAGE_OPTS = [
                help="Image container format to use when creating the image"),
     cfg.StrOpt("name_regex",
                default="^.*(cirros|testvm).*$",
-               help="Regular expression for name of an image to discover it "
-                    "in the cloud and use it for the tests. Note that when "
-                    "Rally is searching for the image, case insensitive "
-                    "matching is performed. Specify nothing ('name_regex =') "
-                    "if you want to disable discovering. In this case Rally "
-                    "will create needed resources by itself if the values "
-                    "for the corresponding config options are not specified "
-                    "in the Tempest config file")
+               help="Regular expression for name of a public image to "
+                    "discover it in the cloud and use it for the tests. "
+                    "Note that when Rally is searching for the image, case "
+                    "insensitive matching is performed. Specify nothing "
+                    "('name_regex =') if you want to disable discovering. "
+                    "In this case Rally will create needed resources by "
+                    "itself if the values for the corresponding config "
+                    "options are not specified in the Tempest config file")
 ]
 
 ROLE_OPTS = [
@@ -240,10 +242,20 @@ class TempestConfig(utils.RandomNameGeneratorMixin):
             self, section_name="network-feature-enabled"):
         if "neutron" in self.available_services:
             neutronclient = self.clients.neutron()
-            ext_list = [ext["alias"] for ext in
-                        neutronclient.list_ext("/extensions")["extensions"]]
-            ext_list_str = ",".join(ext_list)
-            self.conf.set(section_name, "api_extensions", ext_list_str)
+            # NOTE(ylobankov): We need the if/else block here because
+            # the list_ext method has different number of arguments in
+            # different Neutron client versions.
+            cl_ver = nc_version.__version__
+            if version.StrictVersion(cl_ver) >= version.StrictVersion("4.1.0"):
+                # Neutron client version >= 4.1.0
+                extensions = neutronclient.list_ext(
+                    "extensions", "/extensions", retrieve_all=True)
+            else:
+                # Neutron client version < 4.1.0
+                extensions = neutronclient.list_ext("/extensions")
+            aliases = [ext["alias"] for ext in extensions["extensions"]]
+            aliases_str = ",".join(aliases)
+            self.conf.set(section_name, "api_extensions", aliases_str)
 
     def _configure_oslo_concurrency(self, section_name="oslo_concurrency"):
         lock_path = os.path.join(self.data_dir,
@@ -263,8 +275,8 @@ class TempestConfig(utils.RandomNameGeneratorMixin):
         self.conf.set(section_name, "img_file", self.image_name)
 
     def _configure_service_available(self, section_name="service_available"):
-        services = ["ceilometer", "cinder", "glance",
-                    "heat", "neutron", "nova", "sahara", "swift"]
+        services = ["ceilometer", "cinder", "glance", "heat",
+                    "ironic", "neutron", "nova", "sahara", "swift"]
         for service in services:
             # Convert boolean to string because ConfigParser fails
             # on attempt to get option with boolean value
@@ -410,18 +422,20 @@ class TempestResourcesContext(utils.RandomNameGeneratorMixin):
         glance_wrap = glance_wrapper.wrap(self.clients.glance, self)
 
         if CONF.image.name_regex:
-            LOG.debug("Trying to discover an image with name matching "
+            LOG.debug("Trying to discover a public image with name matching "
                       "regular expression '%s'. Note that case insensitive "
                       "matching is performed" % CONF.image.name_regex)
-            img_list = [img for img in self.clients.glance().images.list()
-                        if img.status.lower() == "active" and img.name]
-            for img in img_list:
-                if re.match(CONF.image.name_regex, img.name, re.IGNORECASE):
-                    LOG.debug("The following image discovered: '{0}'. Using "
-                              "image '{0}' for the tests".format(img.name))
+            images = glance_wrap.list_images(status="active",
+                                             visibility="public")
+            for img in images:
+                if img.name and re.match(CONF.image.name_regex,
+                                         img.name, re.IGNORECASE):
+                    LOG.debug(
+                        "The following public image discovered: '{0}'. "
+                        "Using image '{0}' for the tests".format(img.name))
                     return img
 
-            LOG.debug("There is no image with name matching "
+            LOG.debug("There is no public image with name matching "
                       "regular expression '%s'" % CONF.image.name_regex)
 
         params = {
@@ -430,7 +444,7 @@ class TempestResourcesContext(utils.RandomNameGeneratorMixin):
             "container_format": CONF.image.container_format,
             "image_location": os.path.join(_create_or_get_data_dir(),
                                            self.image_name),
-            "is_public": True
+            "visibility": "public"
         }
         LOG.debug("Creating image '%s'" % params["name"])
         image = glance_wrap.create_image(**params)
@@ -467,10 +481,12 @@ class TempestResourcesContext(utils.RandomNameGeneratorMixin):
 
     def _create_network_resources(self):
         neutron_wrapper = network.NeutronWrapper(self.clients, self)
+        tenant_name = self.clients.keystone().tenant_name
+        tenant_id = self.clients.keystone().get_project_id(tenant_name)
         LOG.debug("Creating network resources: network, subnet, router")
         net = neutron_wrapper.create_network(
-            self.clients.keystone().tenant_id, subnets_num=1,
-            add_router=True, network_create_args={"shared": True})
+            tenant_id, subnets_num=1, add_router=True,
+            network_create_args={"shared": True})
         self._created_networks.append(net)
 
         return net
