@@ -16,11 +16,12 @@
 import json
 import os
 import re
+import threading
+import time
 import unittest
 
 import mock
 
-from rally.cmd import envutils
 from tests.functional import utils
 
 
@@ -42,6 +43,63 @@ class TaskTestCase(unittest.TestCase):
             ]
         }
 
+    def _get_sample_task_config_v2(self):
+        return {
+            "version": 2,
+            "title": "Dummy task",
+            "tags": ["dummy", "functional_test"],
+            "subtasks": [
+                {
+                    "title": "first-subtask",
+                    "group": "Dummy group",
+                    "description": "The first subtask in dummy task",
+                    "tags": ["dummy", "functional_test"],
+                    "run_in_parallel": False,
+                    "workloads": [{
+                        "name": "Dummy.dummy",
+                        "args": {
+                            "sleep": 0
+                        },
+                        "runner": {
+                            "type": "constant",
+                            "times": 10,
+                            "concurrency": 2
+                        },
+                        "context": {
+                            "users": {
+                                "tenants": 3,
+                                "users_per_tenant": 2
+                            }
+                        }
+                    }]
+                },
+                {
+                    "title": "second-subtask",
+                    "group": "Dummy group",
+                    "description": "The second subtask in dummy task",
+                    "tags": ["dummy", "functional_test"],
+                    "run_in_parallel": False,
+                    "workloads": [{
+                        "name": "Dummy.dummy",
+                        "args": {
+                            "sleep": 1
+                        },
+                        "runner": {
+                            "type": "constant",
+                            "times": 10,
+                            "concurrency": 2
+                        },
+                        "context": {
+                            "users": {
+                                "tenants": 3,
+                                "users_per_tenant": 2
+                            }
+                        }
+                    }]
+                }
+            ]
+        }
+
     def _get_deployment_uuid(self, output):
         return re.search(
             r"Using deployment: (?P<uuid>[0-9a-f\-]{36})",
@@ -66,6 +124,26 @@ class TaskTestCase(unittest.TestCase):
         self.assertIn("2. dummy_fail_test (2)", detailed_iterations_data)
         self.assertNotIn("n/a", detailed_iterations_data)
 
+    def test_detailed_with_errors(self):
+        rally = utils.Rally()
+        cfg = {
+            "Dummy.dummy_exception": [
+                {
+                    "runner": {
+                        "type": "constant",
+                        "times": 1,
+                        "concurrency": 1
+                    }
+                }
+            ]
+        }
+        config = utils.TaskConfig(cfg)
+        output = rally("task start --task %s" % config.filename)
+        uuid = re.search(
+            r"(?P<uuid>[0-9a-f\-]{36}): started", output).group("uuid")
+        output = rally("task detailed")
+        self.assertIn("Task %s has 1 error(s)" % uuid, output)
+
     def test_detailed_no_atomic_actions(self):
         rally = utils.Rally()
         cfg = {
@@ -95,48 +173,68 @@ class TaskTestCase(unittest.TestCase):
 
     def test_results_with_wrong_task_id(self):
         rally = utils.Rally()
-        self.assertRaises(utils.RallyCmdError,
+        self.assertRaises(utils.RallyCliError,
                           rally, "task results --uuid %s" % FAKE_TASK_UUID)
 
     def test_abort_with_wrong_task_id(self):
         rally = utils.Rally()
-        self.assertRaises(utils.RallyCmdError,
+        self.assertRaises(utils.RallyCliError,
                           rally, "task abort --uuid %s" % FAKE_TASK_UUID)
 
     def test_delete_with_wrong_task_id(self):
         rally = utils.Rally()
-        self.assertRaises(utils.RallyCmdError,
+        self.assertRaises(utils.RallyCliError,
                           rally, "task delete --uuid %s" % FAKE_TASK_UUID)
 
     def test_detailed_with_wrong_task_id(self):
         rally = utils.Rally()
-        self.assertRaises(utils.RallyCmdError,
+        self.assertRaises(utils.RallyCliError,
                           rally, "task detailed --uuid %s" % FAKE_TASK_UUID)
 
     def test_report_with_wrong_task_id(self):
         rally = utils.Rally()
-        self.assertRaises(utils.RallyCmdError,
+        self.assertRaises(utils.RallyCliError,
                           rally, "task report --tasks %s" % FAKE_TASK_UUID)
 
     def test_sla_check_with_wrong_task_id(self):
         rally = utils.Rally()
-        self.assertRaises(utils.RallyCmdError,
+        self.assertRaises(utils.RallyCliError,
                           rally, "task sla_check --uuid %s" % FAKE_TASK_UUID)
 
     def test_status_with_wrong_task_id(self):
         rally = utils.Rally()
-        self.assertRaises(utils.RallyCmdError,
+        self.assertRaises(utils.RallyCliError,
                           rally, "task status --uuid %s" % FAKE_TASK_UUID)
 
+    def _assert_html_report_libs_are_embedded(self, file_path, expected=True):
+
+        embedded_signatures = ["Copyright (c) 2011-2014 Novus Partners, Inc.",
+                               "AngularJS v1.3.3",
+                               "Copyright (c) 2010-2015, Michael Bostock"]
+        external_signatures = ["<script type=\"text/javascript\" src=",
+                               "<link rel=\"stylesheet\" href="]
+        html = open(file_path).read()
+        result_embedded = all([sig in html for sig in embedded_signatures])
+        result_external = all([sig in html for sig in external_signatures])
+        self.assertEqual(expected, result_embedded)
+        self.assertEqual(not expected, result_external)
+
     def test_report_one_uuid(self):
         rally = utils.Rally()
         cfg = self._get_sample_task_config()
         config = utils.TaskConfig(cfg)
         rally("task start --task %s" % config.filename)
         rally("task report --out %s" % rally.gen_report_path(extension="html"))
+        html_report = rally.gen_report_path(extension="html")
+        self.assertTrue(os.path.exists(html_report))
+        self._assert_html_report_libs_are_embedded(html_report, False)
+        self.assertRaises(utils.RallyCliError,
+                          rally, "task report --report %s" % FAKE_TASK_UUID)
+        rally("task report --junit --out %s" %
+              rally.gen_report_path(extension="junit"))
         self.assertTrue(os.path.exists(
-            rally.gen_report_path(extension="html")))
-        self.assertRaises(utils.RallyCmdError,
+            rally.gen_report_path(extension="junit")))
+        self.assertRaises(utils.RallyCliError,
                           rally, "task report --report %s" % FAKE_TASK_UUID)
 
     def test_report_bunch_uuids(self):
@@ -149,10 +247,11 @@ class TaskTestCase(unittest.TestCase):
             for line in res.splitlines():
                 if "finished" in line:
                     task_uuids.append(line.split(" ")[1][:-1])
-        rally("task report --tasks %s --out %s" % (
-              " ".join(task_uuids), rally.gen_report_path(extension="html")))
-        self.assertTrue(os.path.exists(
-            rally.gen_report_path(extension="html")))
+        html_report = rally.gen_report_path(extension="html")
+        rally("task report --tasks %s --out %s" % (" ".join(task_uuids),
+                                                   html_report))
+        self.assertTrue(os.path.exists(html_report))
+        self._assert_html_report_libs_are_embedded(html_report, False)
 
     def test_report_bunch_files(self):
         rally = utils.Rally()
@@ -167,10 +266,11 @@ class TaskTestCase(unittest.TestCase):
                 os.remove(path)
             rally("task results", report_path=path, raw=True)
 
+        html_report = rally.gen_report_path(extension="html")
         rally("task report --tasks %s --out %s" % (
-              " ".join(files), rally.gen_report_path(extension="html")))
-        self.assertTrue(os.path.exists(
-            rally.gen_report_path(extension="html")))
+              " ".join(files), html_report))
+        self.assertTrue(os.path.exists(html_report))
+        self._assert_html_report_libs_are_embedded(html_report, False)
 
     def test_report_one_uuid_one_file(self):
         rally = utils.Rally()
@@ -191,13 +291,24 @@ class TaskTestCase(unittest.TestCase):
         else:
             return 1
 
+        html_report = rally.gen_report_path(extension="html")
         rally("task report --tasks"
               " %s %s --out %s" % (task_result_file, task_uuid,
-                                   rally.gen_report_path(extension="html")))
-        self.assertTrue(os.path.exists(
-            rally.gen_report_path(extension="html")))
-        self.assertRaises(utils.RallyCmdError,
+                                   html_report))
+        self.assertTrue(os.path.exists(html_report))
+        self.assertRaises(utils.RallyCliError,
                           rally, "task report --report %s" % FAKE_TASK_UUID)
+        self._assert_html_report_libs_are_embedded(html_report, False)
+
+    def test_report_one_uuid_with_static_libs(self):
+        rally = utils.Rally()
+        cfg = self._get_sample_task_config()
+        config = utils.TaskConfig(cfg)
+        rally("task start --task %s" % config.filename)
+        html_report = rally.gen_report_path(extension="html")
+        rally("task report --out %s --html-static" % html_report)
+        self.assertTrue(os.path.exists(html_report))
+        self._assert_html_report_libs_are_embedded(html_report)
 
     def test_delete(self):
         rally = utils.Rally()
@@ -228,7 +339,7 @@ class TaskTestCase(unittest.TestCase):
         self.assertIn(
             "deployment_name", rally("task list --all-deployments"))
 
-        self.assertRaises(utils.RallyCmdError,
+        self.assertRaises(utils.RallyCliError,
                           rally, "task list --status not_existing_status")
 
     def test_list_with_print_uuids_option(self):
@@ -270,11 +381,10 @@ class TaskTestCase(unittest.TestCase):
 
     def test_validate_is_invalid(self):
         rally = utils.Rally()
-        with mock.patch.dict("os.environ", utils.TEST_ENV):
-            deployment_id = envutils.get_global("RALLY_DEPLOYMENT")
+        deployment_id = utils.get_global("RALLY_DEPLOYMENT", rally.env)
         cfg = {"invalid": "config"}
         config = utils.TaskConfig(cfg)
-        self.assertRaises(utils.RallyCmdError,
+        self.assertRaises(utils.RallyCliError,
                           rally,
                           ("task validate --task %(task_file)s "
                            "--deployment %(deployment_id)s") %
@@ -283,28 +393,61 @@ class TaskTestCase(unittest.TestCase):
 
     def test_start(self):
         rally = utils.Rally()
-        with mock.patch.dict("os.environ", utils.TEST_ENV):
-            deployment_id = envutils.get_global("RALLY_DEPLOYMENT")
-            cfg = self._get_sample_task_config()
-            config = utils.TaskConfig(cfg)
-            output = rally(("task start --task %(task_file)s "
-                            "--deployment %(deployment_id)s") %
-                           {"task_file": config.filename,
-                            "deployment_id": deployment_id})
+        deployment_id = utils.get_global("RALLY_DEPLOYMENT", rally.env)
+        cfg = self._get_sample_task_config()
+        config = utils.TaskConfig(cfg)
+        output = rally(("task start --task %(task_file)s "
+                        "--deployment %(deployment_id)s") %
+                       {"task_file": config.filename,
+                        "deployment_id": deployment_id})
         result = re.search(
             r"(?P<task_id>[0-9a-f\-]{36}): started", output)
         self.assertIsNotNone(result)
 
+    def test_validate_with_plugin_paths(self):
+        rally = utils.Rally()
+        plugin_paths = ("tests/functional/extra/fake_dir1/,"
+                        "tests/functional/extra/fake_dir2/")
+        task_file = "tests/functional/extra/test_fake_scenario.json"
+        output = rally(("--plugin-paths %(plugin_paths)s "
+                        "task validate --task %(task_file)s") %
+                       {"task_file": task_file,
+                        "plugin_paths": plugin_paths})
+
+        self.assertIn("Task config is valid", output)
+
+        plugin_paths = ("tests/functional/extra/fake_dir1/"
+                        "fake_plugin1.py,"
+                        "tests/functional/extra/fake_dir2/"
+                        "fake_plugin2.py")
+        task_file = "tests/functional/extra/test_fake_scenario.json"
+        output = rally(("--plugin-paths %(plugin_paths)s "
+                        "task validate --task %(task_file)s") %
+                       {"task_file": task_file,
+                        "plugin_paths": plugin_paths})
+
+        self.assertIn("Task config is valid", output)
+
+        plugin_paths = ("tests/functional/extra/fake_dir1/,"
+                        "tests/functional/extra/fake_dir2/"
+                        "fake_plugin2.py")
+        task_file = "tests/functional/extra/test_fake_scenario.json"
+        output = rally(("--plugin-paths %(plugin_paths)s "
+                        "task validate --task %(task_file)s") %
+                       {"task_file": task_file,
+                        "plugin_paths": plugin_paths})
+
+        self.assertIn("Task config is valid", output)
+
     def _test_start_abort_on_sla_failure_success(self, cfg, times):
         rally = utils.Rally()
-        with mock.patch.dict("os.environ", utils.TEST_ENV):
-            deployment_id = envutils.get_global("RALLY_DEPLOYMENT")
-            config = utils.TaskConfig(cfg)
-            rally(("task start --task %(task_file)s "
-                   "--deployment %(deployment_id)s --abort-on-sla-failure") %
-                  {"task_file": config.filename,
-                   "deployment_id": deployment_id})
-            results = json.loads(rally("task results"))
+        deployment_id = utils.get_global("RALLY_DEPLOYMENT", rally.env)
+        config = utils.TaskConfig(cfg)
+        rally(("task start --task %(task_file)s "
+               "--deployment %(deployment_id)s --abort-on-sla-failure") %
+              {"task_file": config.filename,
+               "deployment_id": deployment_id})
+        results = json.loads(rally("task results"))
         iterations_completed = len(results[0]["result"])
         self.assertEqual(times, iterations_completed)
 
@@ -372,14 +515,13 @@ class TaskTestCase(unittest.TestCase):
 
     def _test_start_abort_on_sla_failure(self, cfg, times):
         rally = utils.Rally()
-        with mock.patch.dict("os.environ", utils.TEST_ENV):
-            deployment_id = envutils.get_global("RALLY_DEPLOYMENT")
-            config = utils.TaskConfig(cfg)
-            rally(("task start --task %(task_file)s "
-                   "--deployment %(deployment_id)s --abort-on-sla-failure") %
-                  {"task_file": config.filename,
-                   "deployment_id": deployment_id})
-            results = json.loads(rally("task results"))
+        deployment_id = utils.get_global("RALLY_DEPLOYMENT", rally.env)
+        config = utils.TaskConfig(cfg)
+        rally(("task start --task %(task_file)s "
+               "--deployment %(deployment_id)s --abort-on-sla-failure") %
+              {"task_file": config.filename,
+               "deployment_id": deployment_id})
+        results = json.loads(rally("task results"))
         iterations_completed = len(results[0]["result"])
         self.assertTrue(iterations_completed < times)
 
@@ -507,25 +649,169 @@ class TaskTestCase(unittest.TestCase):
         }
         self._test_start_abort_on_sla_failure(cfg, times)
 
-    # NOTE(oanufriev): Not implemented
+    def _start_task_in_new_thread(self, rally, cfg, report_file):
+        deployment_id = utils.get_global("RALLY_DEPLOYMENT", rally.env)
+        config = utils.TaskConfig(cfg)
+        cmd = (("task start --task %(task_file)s "
+                "--deployment %(deployment_id)s") %
+               {"task_file": config.filename,
+                "deployment_id": deployment_id})
+        report_path = os.path.join(
+            os.environ.get("REPORTS_ROOT", "rally-cli-output-files"),
+            "TaskTestCase", report_file)
+        task = threading.Thread(target=rally, args=(cmd, ),
+                                kwargs={"report_path": report_path})
+        task.start()
+        uuid = None
+        while not uuid:
+            if not uuid:
+                uuid = utils.get_global("RALLY_TASK", rally.env)
+                time.sleep(0.5)
+        return task, uuid
+
     def test_abort(self):
-        pass
+        RUNNER_TIMES = 10
+        cfg = {
+            "Dummy.dummy": [
+                {
+                    "args": {
+                        "sleep": 5
+                    },
+                    "runner": {
+                        "type": "serial",
+                        "times": RUNNER_TIMES
+                    }
+                }
+            ]
+        }
+        rally = utils.Rally()
+        task, uuid = self._start_task_in_new_thread(
+            rally, cfg, "test_abort-thread_with_abort.txt")
+        rally("task abort %s" % uuid)
+        task.join()
+        results = json.loads(rally("task results"))
+        iterations_completed = len(results[0]["result"])
+        # NOTE(msdubov): check that the task is really stopped before
+        #                the specified number of iterations
+        self.assertTrue(iterations_completed < RUNNER_TIMES)
+        self.assertIn("aborted", rally("task status"))
+        report = rally.gen_report_path(extension="html")
+        rally("task report --out %s" % report)
+
+    def test_abort_soft(self):
+        cfg = {
+            "Dummy.dummy": [
+                {
+                    "args": {
+                        "sleep": 2
+                    },
+                    "runner": {
+                        "type": "serial",
+                        "times": 3,
+                    }
+                },
+                {
+                    "runner": {
+                        "type": "serial",
+                        "times": 10,
+                    }
+                }
+            ]
+        }
+        rally = utils.Rally()
+        task, uuid = self._start_task_in_new_thread(
+            rally, cfg, "test_abort_soft-thread_with_soft_abort.txt")
+        rally("task abort --soft")
+        task.join()
+        results = json.loads(rally("task results"))
+        iterations_completed = len(results[0]["result"])
+        # NOTE(msdubov): check that the task is stopped after first runner
+        #                benchmark finished all its iterations
+        self.assertEqual(3, iterations_completed)
+        # NOTE(msdubov): check that the next benchmark scenario is not started
+        self.assertEqual(1, len(results))
+        self.assertIn("aborted", rally("task status"))
 
     def test_use(self):
         rally = utils.Rally()
-        with mock.patch.dict("os.environ", utils.TEST_ENV):
-            deployment_id = envutils.get_global("RALLY_DEPLOYMENT")
-            config = utils.TaskConfig(self._get_sample_task_config())
-            output = rally(("task start --task %(task_file)s "
-                            "--deployment %(deployment_id)s") %
-                           {"task_file": config.filename,
-                            "deployment_id": deployment_id})
-            result = re.search(
-                r"(?P<uuid>[0-9a-f\-]{36}): started", output)
-            uuid = result.group("uuid")
-            rally("task use --task %s" % uuid)
-            current_task = envutils.get_global("RALLY_TASK")
-            self.assertEqual(uuid, current_task)
+        deployment_id = utils.get_global("RALLY_DEPLOYMENT", rally.env)
+        config = utils.TaskConfig(self._get_sample_task_config())
+        output = rally(("task start --task %(task_file)s "
+                        "--deployment %(deployment_id)s") %
+                       {"task_file": config.filename,
+                        "deployment_id": deployment_id})
+        result = re.search(
+            r"(?P<uuid>[0-9a-f\-]{36}): started", output)
+        uuid = result.group("uuid")
+        rally("task use --task %s" % uuid)
+        current_task = utils.get_global("RALLY_TASK", rally.env)
+        self.assertEqual(uuid, current_task)
+
+    def test_start_v2(self):
+        rally = utils.Rally()
+        deployment_id = utils.get_global("RALLY_DEPLOYMENT", rally.env)
+        cfg = self._get_sample_task_config_v2()
+        config = utils.TaskConfig(cfg)
+        output = rally(("task start --task %(task_file)s "
+                        "--deployment %(deployment_id)s") %
+                       {"task_file": config.filename,
+                        "deployment_id": deployment_id})
+        result = re.search(
+            r"(?P<task_id>[0-9a-f\-]{36}): started", output)
+        self.assertIsNotNone(result)
+
+    def test_export(self):
+        rally = utils.Rally()
+        cfg = {
+            "Dummy.dummy": [
+                {
+                    "runner": {
+                        "type": "constant",
+                        "times": 100,
+                        "concurrency": 5
+                    }
+                }
+            ]
+        }
+        config = utils.TaskConfig(cfg)
+        output = rally("task start --task %s" % config.filename)
+        uuid = re.search(
+            r"(?P<uuid>[0-9a-f\-]{36}): started", output).group("uuid")
+        connection = (
+            "file-exporter:///" + rally.gen_report_path(extension="json"))
+        output = rally("task export --uuid %s --connection %s" % (
+            uuid, connection))
+        expected = (
+            "Task %(uuid)s results was successfully exported to %("
+            "connection)s using file-exporter plugin." % {
+                "uuid": uuid,
+                "connection": connection,
+            })
+        self.assertIn(expected, output)
+
+    def test_export_with_wrong_connection(self):
+        rally = utils.Rally()
+        cfg = {
+            "Dummy.dummy": [
+                {
+                    "runner": {
+                        "type": "constant",
+                        "times": 100,
+                        "concurrency": 5
+                    }
+                }
+            ]
+        }
+        config = utils.TaskConfig(cfg)
+        output = rally("task start --task %s" % config.filename)
+        uuid = re.search(
+            r"(?P<uuid>[0-9a-f\-]{36}): started", output).group("uuid")
+        connection = (
+            "fake:///" + rally.gen_report_path(extension="json"))
+        self.assertRaises(utils.RallyCliError,
+                          rally,
+                          "task export --uuid %s --connection %s" % (
+                              uuid, connection))
 
 
 class SLATestCase(unittest.TestCase):
@@ -536,7 +822,7 @@ class SLATestCase(unittest.TestCase):
             "KeystoneBasic.create_and_list_users": [
                 {
                     "args": {
-                        "name_length": 10
+                        "enabled": True
                     },
                     "runner": {
                         "type": "constant",
@@ -556,7 +842,7 @@ class SLATestCase(unittest.TestCase):
         cfg = self._get_sample_task_config(max_seconds_per_iteration=0.001)
         config = utils.TaskConfig(cfg)
         rally("task start --task %s" % config.filename)
-        self.assertRaises(utils.RallyCmdError, rally, "task sla_check")
+        self.assertRaises(utils.RallyCliError, rally, "task sla_check")
 
     def test_sla_success(self):
         rally = utils.Rally()
@@ -609,7 +895,7 @@ class SLAExtraFlagsTestCase(unittest.TestCase):
         ]
         try:
             rally("task sla_check --json", getjson=True)
-        except utils.RallyCmdError as expected_error:
+        except utils.RallyCliError as expected_error:
             self.assertEqual(json.loads(expected_error.output), expected)
         else:
             self.fail("`rally task sla_check` command should return non-zero "
@@ -637,7 +923,7 @@ class SLAExtraFlagsTestCase(unittest.TestCase):
         ]
         try:
             rally("task sla_check --json", getjson=True)
-        except utils.RallyCmdError as expected_error:
+        except utils.RallyCliError as expected_error:
             self.assertEqual(json.loads(expected_error.output), expected)
         else:
             self.fail("`rally task sla_check` command should return non-zero "
