@@ -14,10 +14,13 @@
 #    under the License.
 
 import copy
-import datetime as date
+import datetime as dt
+import json
 import os.path
 
+import ddt
 import mock
+import yaml
 
 from rally.cli.commands import task
 from rally import consts
@@ -26,6 +29,7 @@ from tests.unit import fakes
 from tests.unit import test
 
 
+@ddt.ddt
 class TaskCommandsTestCase(test.TestCase):
 
     def setUp(self):
@@ -33,7 +37,7 @@ class TaskCommandsTestCase(test.TestCase):
         self.task = task.TaskCommands()
 
     @mock.patch("rally.cli.commands.task.open", create=True)
-    def test_load_task(self, mock_open):
+    def test__load_task(self, mock_open):
         input_task = "{'ab': {{test}}}"
         input_args = "{'test': 2}"
 
@@ -43,26 +47,28 @@ class TaskCommandsTestCase(test.TestCase):
             mock.mock_open(read_data="{'test': 1}").return_value,
             mock.mock_open(read_data=input_task).return_value
         ]
-        result = self.task._load_task("in_task", task_args_file="in_args_path")
-        self.assertEqual(result, {"ab": 1})
+        task_conf = self.task._load_task(
+            "in_task", task_args_file="in_args_path")
+        self.assertEqual({"ab": 1}, task_conf)
 
         mock_open.side_effect = [
             mock.mock_open(read_data=input_task).return_value
         ]
-        result = self.task._load_task("in_task", task_args=input_args)
-        self.assertEqual(result, {"ab": 2})
+        task_conf = self.task._load_task(
+            "in_task", task_args=input_args)
+        self.assertEqual(task_conf, {"ab": 2})
 
         mock_open.side_effect = [
             mock.mock_open(read_data="{'test': 1}").return_value,
             mock.mock_open(read_data=input_task).return_value
 
         ]
-        result = self.task._load_task("in_task", task_args=input_args,
-                                      task_args_file="any_file")
-        self.assertEqual(result, {"ab": 2})
+        task_conf = self.task._load_task(
+            "in_task", task_args=input_args, task_args_file="any_file")
+        self.assertEqual(task_conf, {"ab": 2})
 
     @mock.patch("rally.cli.commands.task.open", create=True)
-    def test_load_task_wrong_task_args_file(self, mock_open):
+    def test__load_task_wrong_task_args_file(self, mock_open):
         mock_open.side_effect = [
             mock.mock_open(read_data="{'test': {}").return_value
         ]
@@ -71,19 +77,20 @@ class TaskCommandsTestCase(test.TestCase):
                           "in_task", task_args_file="in_args_path")
 
     @mock.patch("rally.cli.commands.task.open", create=True)
-    def test_load_task_wrong_task_args_file_exception(self, mock_open):
+    def test__load_task_wrong_task_args_file_exception(self, mock_open):
         mock_open.side_effect = IOError
         self.assertRaises(IOError, self.task._load_task,
                           "in_task", task_args_file="in_args_path")
 
-    def test_load_task_wrong_input_task_args(self):
+    def test__load_task_wrong_input_task_args(self):
         self.assertRaises(task.FailedToLoadTask,
-                          self.task._load_task, "in_task", "{'test': {}")
+                          self.task._load_task, "in_task",
+                          "{'test': {}")
         self.assertRaises(task.FailedToLoadTask,
                           self.task._load_task, "in_task", "[]")
 
     @mock.patch("rally.cli.commands.task.open", create=True)
-    def test_load_task_task_render_raise_exc(self, mock_open):
+    def test__load_task_task_render_raise_exc(self, mock_open):
         mock_open.side_effect = [
             mock.mock_open(read_data="{'test': {{t}}}").return_value
         ]
@@ -91,7 +98,7 @@ class TaskCommandsTestCase(test.TestCase):
                           self.task._load_task, "in_task")
 
     @mock.patch("rally.cli.commands.task.open", create=True)
-    def test_load_task_task_not_in_yaml(self, mock_open):
+    def test__load_task_task_not_in_yaml(self, mock_open):
         mock_open.side_effect = [
             mock.mock_open(read_data="{'test': {}").return_value
         ]
@@ -116,35 +123,84 @@ class TaskCommandsTestCase(test.TestCase):
             actual = self.task._load_task(input_task_file)
         self.assertEqual(expect, actual)
 
+    @mock.patch("rally.cli.commands.task.os.path.isfile", return_value=True)
+    @mock.patch("rally.cli.commands.task.api.Task.validate",
+                return_value=fakes.FakeTask())
+    @mock.patch("rally.cli.commands.task.TaskCommands._load_task",
+                return_value={"uuid": "some_uuid"})
+    def test__load_and_validate_task(self, mock__load_task,
+                                     mock_task_validate, mock_os_path_isfile):
+        deployment = "some_deployment_uuid"
+        self.task._load_and_validate_task("some_task", "task_args",
+                                          "task_args_file", deployment)
+        mock__load_task.assert_called_once_with("some_task", "task_args",
+                                                "task_args_file")
+        mock_task_validate.assert_called_once_with(
+            deployment, mock__load_task.return_value, None)
+
+    @mock.patch("rally.cli.commands.task.os.path.isfile", return_value=False)
+    @mock.patch("rally.cli.commands.task.TaskCommands._load_task")
+    @mock.patch("rally.api.Task.validate")
+    def test__load_and_validate_file(self, mock_task_validate, mock__load_task,
+                                     mock_os_path_isfile):
+        deployment = "some_deployment_uuid"
+        self.assertRaises(IOError, self.task._load_and_validate_task,
+                          "some_task", "task_args",
+                          "task_args_file", deployment)
+
+    @mock.patch("rally.cli.commands.task.os.path.isfile", return_value=True)
+    @mock.patch("rally.cli.commands.task.api.Task.create",
+                return_value=fakes.FakeTask(uuid="some_new_uuid", tag="tag"))
+    @mock.patch("rally.cli.commands.task.TaskCommands.use")
     @mock.patch("rally.cli.commands.task.TaskCommands.detailed")
     @mock.patch("rally.cli.commands.task.TaskCommands._load_task",
                 return_value={"some": "json"})
-    @mock.patch("rally.api.Task.create")
+    @mock.patch("rally.cli.commands.task.api.Task.validate",
+                return_value=fakes.FakeTask(some="json", uuid="some_uuid",
+                                            temporary=True))
     @mock.patch("rally.cli.commands.task.api.Task.start")
-    def test_start(self, mock_task_start, mock_task_create, mock__load_task,
-                   mock_task_commands_detailed):
-        mock_task_create.return_value = (
-            dict(uuid="c1a9bbe-1ead-4740-92b5-0feecf421634",
-                 created_at="2014-01-14 09:14:45.395822",
-                 status="init", tag=None))
+    def test_start(self, mock_task_start, mock_task_validate, mock__load_task,
+                   mock_detailed, mock_use, mock_task_create,
+                   mock_os_path_isfile):
         deployment_id = "e0617de9-77d1-4875-9b49-9d5789e29f20"
         task_path = "path_to_config.json"
-        self.task.start(task_path, deployment_id)
+        self.task.start(task_path, deployment_id, do_use=True)
+        mock_task_create.assert_called_once_with(
+            deployment_id, None)
         mock_task_start.assert_called_once_with(
-            deployment_id, {"some": "json"},
-            task=mock_task_create.return_value, abort_on_sla_failure=False)
+            deployment_id, mock__load_task.return_value,
+            task=mock_task_validate.return_value, abort_on_sla_failure=False)
         mock__load_task.assert_called_once_with(task_path, None, None)
+        mock_use.assert_called_once_with("some_new_uuid")
+        mock_detailed.assert_called_once_with(task_id="some_new_uuid")
 
+    @mock.patch("rally.cli.commands.task.os.path.isfile", return_value=True)
+    @mock.patch("rally.cli.commands.task.api.Task.create",
+                return_value=fakes.FakeTask(uuid="new_uuid", tag="some_tag"))
+    @mock.patch("rally.cli.commands.task.TaskCommands.detailed")
+    @mock.patch("rally.cli.commands.task.api.Task.start")
     @mock.patch("rally.cli.commands.task.TaskCommands._load_task",
-                side_effect=task.FailedToLoadTask)
-    def test_start_with_task_args(self, mock__load_task):
+                return_value="some_config")
+    @mock.patch("rally.cli.commands.task.api.Task.validate",
+                return_value=fakes.FakeTask(uuid="some_id"))
+    def test_start_with_task_args(self, mock_task_validate, mock__load_task,
+                                  mock_task_start, mock_detailed,
+                                  mock_task_create, mock_os_path_isfile):
         task_path = mock.MagicMock()
         task_args = mock.MagicMock()
         task_args_file = mock.MagicMock()
         self.task.start(task_path, deployment="any", task_args=task_args,
-                        task_args_file=task_args_file)
-        mock__load_task.assert_called_once_with(
-            task_path, task_args, task_args_file)
+                        task_args_file=task_args_file, tag="some_tag")
+        mock__load_task.assert_called_once_with(task_path, task_args,
+                                                task_args_file)
+        mock_task_validate.assert_called_once_with(
+            "any", mock__load_task.return_value, {})
+        mock_task_start.assert_called_once_with(
+            "any", mock__load_task.return_value,
+            task=mock_task_create.return_value, abort_on_sla_failure=False)
+        mock_detailed.assert_called_once_with(
+            task_id=mock_task_create.return_value["uuid"])
+        mock_task_create.assert_called_once_with("any", "some_tag")
 
     @mock.patch("rally.cli.commands.task.envutils.get_global")
     def test_start_no_deployment_id(self, mock_get_global):
@@ -152,18 +208,26 @@ class TaskCommandsTestCase(test.TestCase):
         self.assertRaises(exceptions.InvalidArgumentsException,
                           self.task.start, "path_to_config.json", None)
 
-    @mock.patch("rally.cli.commands.task.TaskCommands._load_task")
-    @mock.patch("rally.cli.commands.task.api")
-    def test_start_invalid_task(self, mock_api, mock__load_task):
-        mock_api.Task.start.side_effect = exceptions.InvalidConfigException
-
+    @mock.patch("rally.cli.commands.task.os.path.isfile", return_value=True)
+    @mock.patch("rally.cli.commands.task.api.Task.create",
+                return_value=fakes.FakeTask(temporary=False, tag="tag",
+                                            uuid="uuid"))
+    @mock.patch("rally.cli.commands.task.TaskCommands._load_task",
+                return_value={"some": "json"})
+    @mock.patch("rally.cli.commands.task.api.Task.validate")
+    @mock.patch("rally.cli.commands.task.api.Task.start",
+                side_effect=exceptions.InvalidTaskException)
+    def test_start_invalid_task(self, mock_task_start, mock_task_validate,
+                                mock__load_task, mock_task_create,
+                                mock_os_path_isfile):
         result = self.task.start("task_path", "deployment", tag="tag")
         self.assertEqual(1, result)
 
-        mock_api.Task.create.assert_called_once_with("deployment", "tag")
-        mock_api.Task.start.assert_called_once_with(
+        mock_task_create.assert_called_once_with("deployment", "tag")
+
+        mock_task_start.assert_called_once_with(
             "deployment", mock__load_task.return_value,
-            task=mock_api.Task.create.return_value, abort_on_sla_failure=False)
+            task=mock_task_create.return_value, abort_on_sla_failure=False)
 
     @mock.patch("rally.cli.commands.task.api")
     def test_abort(self, mock_api):
@@ -182,10 +246,10 @@ class TaskCommandsTestCase(test.TestCase):
     def test_status(self):
         test_uuid = "a3e7cefb-bec2-4802-89f6-410cc31f71af"
         value = {"task_id": "task", "status": "status"}
-        with mock.patch("rally.cli.commands.task.db") as mock_db:
-            mock_db.task_get = mock.MagicMock(return_value=value)
+        with mock.patch("rally.cli.commands.task.api.Task") as mock_task:
+            mock_task.get = mock.MagicMock(return_value=value)
             self.task.status(test_uuid)
-            mock_db.task_get.assert_called_once_with(test_uuid)
+            mock_task.get.assert_called_once_with(test_uuid)
 
     @mock.patch("rally.cli.commands.task.envutils.get_global")
     def test_status_no_task_id(self, mock_get_global):
@@ -193,10 +257,10 @@ class TaskCommandsTestCase(test.TestCase):
         self.assertRaises(exceptions.InvalidArgumentsException,
                           self.task.status, None)
 
-    @mock.patch("rally.cli.commands.task.db")
-    def test_detailed(self, mock_db):
+    @mock.patch("rally.cli.commands.task.api.Task")
+    def test_detailed(self, mock_task):
         test_uuid = "c0d874d4-7195-4fd5-8688-abe82bfad36f"
-        value = {
+        mock_task.get_detailed.return_value = {
             "id": "task",
             "uuid": test_uuid,
             "status": "status",
@@ -207,83 +271,101 @@ class TaskCommandsTestCase(test.TestCase):
                         "pos": "fake_pos",
                         "kw": "fake_kw"
                     },
-                    "data": {
-                        "load_duration": 1.0,
-                        "full_duration": 2.0,
-                        "raw": [
-                            {
-                                "duration": 0.9,
-                                "idle_duration": 0.5,
-                                "scenario_output": {
-                                    "data": {
-                                        "a": 3
-                                    },
-                                    "errors": "some"
-                                },
-                                "atomic_actions": {
-                                    "a": 0.6,
-                                    "b": 0.7
-                                },
-                                "error": ["type", "message", "traceback"]
-                            },
-                            {
-                                "duration": 0.5,
-                                "idle_duration": 0.2,
-                                "scenario_output": {
-                                    "data": {
-                                        "a": 1
-                                    },
-                                    "errors": "some"
-                                },
-                                "atomic_actions": {
-                                    "a": 0.2,
-                                    "b": 0.4
-                                },
-                                "error": None
-                            },
-                            {
-                                "duration": 0.6,
-                                "idle_duration": 0.4,
-                                "scenario_output": {
-                                    "data": {
-                                        "a": 2
-                                    },
-                                    "errors": None
-                                },
-                                "atomic_actions": {
-                                    "a": 0.3,
-                                    "b": 0.5
-                                },
-                                "error": None
-                            }
-                        ]
-                    }
+                    "info": {
+                        "load_duration": 3.2,
+                        "full_duration": 3.5,
+                        "iterations_count": 4,
+                        "atomic": {"foo": {}, "bar": {}}},
+                    "iterations": [
+                        {"duration": 0.9,
+                         "idle_duration": 0.1,
+                         "output": {"additive": [], "complete": []},
+                         "atomic_actions": {"foo": 0.6, "bar": 0.7},
+                         "error": ["type", "message", "traceback"]
+                         },
+                        {"duration": 1.2,
+                         "idle_duration": 0.3,
+                         "output": {"additive": [], "complete": []},
+                            "atomic_actions": {"foo": 0.6, "bar": 0.7},
+                            "error": ["type", "message", "traceback"]
+                         },
+                        {"duration": 0.7,
+                         "idle_duration": 0.5,
+                         "scenario_output": {
+                             "data": {"foo": 0.6, "bar": 0.7},
+                             "errors": "some"
+                         },
+                         "atomic_actions": {"foo": 0.6, "bar": 0.7},
+                         "error": ["type", "message", "traceback"]
+                         },
+                        {"duration": 0.5,
+                         "idle_duration": 0.5,
+                         "output": {"additive": [], "complete": []},
+                         "atomic_actions": {"foo": 0.6, "bar": 0.7},
+                         "error": ["type", "message", "traceback"]
+                         }
+                    ]
                 }
             ]
         }
-        mock_db.task_get_detailed = mock.MagicMock(return_value=value)
         self.task.detailed(test_uuid)
-        mock_db.task_get_detailed.assert_called_once_with(test_uuid)
-
+        mock_task.get_detailed.assert_called_once_with(test_uuid,
+                                                       extended_results=True)
         self.task.detailed(test_uuid, iterations_data=True)
 
-    @mock.patch("rally.cli.commands.task.db")
+    @mock.patch("rally.cli.commands.task.sys.stdout")
+    @mock.patch("rally.cli.commands.task.api.Task")
     @mock.patch("rally.cli.commands.task.logging")
-    def test_detailed_task_failed(self, mock_logging, mock_db):
+    @ddt.data({"debug": True},
+              {"debug": False})
+    @ddt.unpack
+    def test_detailed_task_failed(self, mock_logging, mock_task,
+                                  mock_stdout, debug):
+        test_uuid = "test_task_id"
         value = {
             "id": "task",
-            "uuid": "task_uuid",
+            "uuid": test_uuid,
             "status": consts.TaskStatus.FAILED,
             "results": [],
-            "verification_log": "['1', '2', '3']"
+            "verification_log": json.dumps(["error_type", "error_message",
+                                            "error_traceback"])
         }
-        mock_db.task_get_detailed = mock.MagicMock(return_value=value)
+        mock_task.get_detailed = mock.MagicMock(return_value=value)
 
-        mock_logging.is_debug.return_value = False
-        self.task.detailed("task_uuid")
+        mock_logging.is_debug.return_value = debug
+        self.task.detailed(test_uuid)
+        verification = yaml.safe_load(value["verification_log"])
+        if debug:
+            expected_calls = [mock.call("Task test_task_id: failed"),
+                              mock.call("%s" % verification[2])]
+            mock_stdout.write.assert_has_calls(expected_calls, any_order=True)
+        else:
+            expected_calls = [mock.call("Task test_task_id: failed"),
+                              mock.call("%s" % verification[0]),
+                              mock.call("%s" % verification[1]),
+                              mock.call("\nFor more details run:\nrally "
+                                        "-vd task detailed %s" % test_uuid)]
+            mock_stdout.write.assert_has_calls(expected_calls, any_order=True)
 
-        mock_logging.is_debug.return_value = True
-        self.task.detailed("task_uuid")
+    @mock.patch("rally.cli.commands.task.api.Task")
+    @mock.patch("rally.cli.commands.task.sys.stdout")
+    def test_detailed_task_status_not_in_finished_abort(self,
+                                                        mock_stdout,
+                                                        mock_task):
+        test_uuid = "test_task_id"
+        value = {
+            "id": "task",
+            "uuid": test_uuid,
+            "status": consts.TaskStatus.INIT,
+            "results": []
+        }
+        mock_task.get_detailed = mock.MagicMock(return_value=value)
+        self.task.detailed(test_uuid)
+        expected_calls = [mock.call("Task test_task_id: init"),
+                          mock.call("\nThe task test_task_id marked as "
+                                    "'init'. Results available when it "
+                                    "is 'finished'.")]
+        mock_stdout.write.assert_has_calls(expected_calls, any_order=True)
 
     @mock.patch("rally.cli.commands.task.envutils.get_global")
     def test_detailed_no_task_id(self, mock_get_global):
@@ -291,29 +373,31 @@ class TaskCommandsTestCase(test.TestCase):
         self.assertRaises(exceptions.InvalidArgumentsException,
                           self.task.detailed, None)
 
-    @mock.patch("rally.cli.commands.task.db")
-    def test_detailed_wrong_id(self, mock_db):
+    @mock.patch("rally.cli.commands.task.api.Task")
+    def test_detailed_wrong_id(self, mock_task):
         test_uuid = "eb290c30-38d8-4c8f-bbcc-fc8f74b004ae"
-        mock_db.task_get_detailed = mock.MagicMock(return_value=None)
+        mock_task.get_detailed = mock.MagicMock(return_value=None)
         self.task.detailed(test_uuid)
-        mock_db.task_get_detailed.assert_called_once_with(test_uuid)
+        mock_task.get_detailed.assert_called_once_with(test_uuid,
+                                                       extended_results=True)
 
     @mock.patch("json.dumps")
-    @mock.patch("rally.cli.commands.task.objects.Task.get")
+    @mock.patch("rally.cli.commands.task.api.Task.get")
     def test_results(self, mock_task_get, mock_json_dumps):
         task_id = "foo_task_id"
         data = [
             {"key": "foo_key", "data": {"raw": "foo_raw", "sla": [],
-                                        "load_duration": "lo_duration",
-                                        "full_duration": "fu_duration"}}
+                                        "load_duration": 1.0,
+                                        "full_duration": 2.0}}
         ]
         result = map(lambda x: {"key": x["key"],
                                 "result": x["data"]["raw"],
                                 "load_duration": x["data"]["load_duration"],
                                 "full_duration": x["data"]["full_duration"],
                                 "sla": x["data"]["sla"]}, data)
-        mock_results = mock.Mock(return_value=data)
-        mock_task_get.return_value = mock.Mock(get_results=mock_results)
+        fake_task = fakes.FakeTask({"status": consts.TaskStatus.FINISHED})
+        fake_task.get_results = mock.MagicMock(return_value=data)
+        mock_task_get.return_value = fake_task
 
         self.task.results(task_id)
         self.assertEqual(1, mock_json_dumps.call_count)
@@ -324,17 +408,21 @@ class TaskCommandsTestCase(test.TestCase):
         mock_task_get.assert_called_once_with(task_id)
 
     @mock.patch("rally.cli.commands.task.sys.stdout")
-    @mock.patch("rally.cli.commands.task.objects.Task.get")
+    @mock.patch("rally.cli.commands.task.api.Task.get")
     def test_results_no_data(self, mock_task_get, mock_stdout):
         task_id = "foo_task_id"
-        mock_results = mock.Mock(return_value=[])
-        mock_task_get.return_value = mock.Mock(get_results=mock_results)
+        fake_task = fakes.FakeTask({"status": consts.TaskStatus.FAILED})
+        mock_task_get.return_value = fake_task
+
+        self.assertEqual(1, self.task.results(task_id))
 
-        result = self.task.results(task_id)
         mock_task_get.assert_called_once_with(task_id)
-        self.assertEqual(1, result)
-        expected_out = ("The task %s is still running, results will become"
-                        " available when it is finished." % task_id)
+
+        expected_out = ("Task status is %s. Results "
+                        "available when it is one of %s.") % (
+            consts.TaskStatus.FAILED,
+            ", ".join((consts.TaskStatus.FINISHED,
+                       consts.TaskStatus.ABORTED)))
         mock_stdout.write.assert_has_calls([mock.call(expected_out)])
 
     @mock.patch("rally.cli.commands.task.jsonschema.validate",
@@ -345,7 +433,7 @@ class TaskCommandsTestCase(test.TestCase):
                 side_effect=mock.mock_open(), create=True)
     @mock.patch("rally.cli.commands.task.plot")
     @mock.patch("rally.cli.commands.task.webbrowser")
-    @mock.patch("rally.cli.commands.task.objects.Task.get")
+    @mock.patch("rally.cli.commands.task.api.Task.get")
     def test_report_one_uuid(self, mock_task_get, mock_webbrowser,
                              mock_plot, mock_open, mock_realpath,
                              mock_validate):
@@ -375,20 +463,32 @@ class TaskCommandsTestCase(test.TestCase):
                 m.reset_mock()
         self.task.report(tasks=task_id, out="/tmp/%s.html" % task_id)
         mock_open.assert_called_once_with("/tmp/%s.html" % task_id, "w+")
-        mock_plot.plot.assert_called_once_with(results)
+        mock_plot.plot.assert_called_once_with(results, include_libs=False)
 
         mock_open.side_effect().write.assert_called_once_with("html_report")
         mock_task_get.assert_called_once_with(task_id)
 
+        # JUnit
         reset_mocks()
         self.task.report(tasks=task_id, out="/tmp/%s.html" % task_id,
                          out_format="junit")
         mock_open.assert_called_once_with("/tmp/%s.html" % task_id, "w+")
+        self.assertFalse(mock_plot.plot.called)
 
+        # HTML
         reset_mocks()
-        self.task.report(task_id, out="spam.html", open_it=True)
+        self.task.report(task_id, out="output.html", open_it=True,
+                         out_format="html")
         mock_webbrowser.open_new_tab.assert_called_once_with(
-            "file://realpath_spam.html")
+            "file://realpath_output.html")
+        mock_plot.plot.assert_called_once_with(results, include_libs=False)
+
+        # HTML with embedded JS/CSS
+        reset_mocks()
+        self.task.report(task_id, open_it=False, out="output.html",
+                         out_format="html_static")
+        self.assertFalse(mock_webbrowser.open_new_tab.called)
+        mock_plot.plot.assert_called_once_with(results, include_libs=True)
 
     @mock.patch("rally.cli.commands.task.jsonschema.validate",
                 return_value=None)
@@ -398,7 +498,7 @@ class TaskCommandsTestCase(test.TestCase):
                 side_effect=mock.mock_open(), create=True)
     @mock.patch("rally.cli.commands.task.plot")
     @mock.patch("rally.cli.commands.task.webbrowser")
-    @mock.patch("rally.cli.commands.task.objects.Task.get")
+    @mock.patch("rally.cli.commands.task.api.Task.get")
     def test_report_bunch_uuids(self, mock_task_get, mock_webbrowser,
                                 mock_plot, mock_open, mock_realpath,
                                 mock_validate):
@@ -433,7 +533,7 @@ class TaskCommandsTestCase(test.TestCase):
                 m.reset_mock()
         self.task.report(tasks=tasks, out="/tmp/1_test.html")
         mock_open.assert_called_once_with("/tmp/1_test.html", "w+")
-        mock_plot.plot.assert_called_once_with(results)
+        mock_plot.plot.assert_called_once_with(results, include_libs=False)
 
         mock_open.side_effect().write.assert_called_once_with("html_report")
         expected_get_calls = [mock.call(task) for task in tasks]
@@ -480,7 +580,7 @@ class TaskCommandsTestCase(test.TestCase):
         expected_open_calls = [mock.call(task_file, "r"),
                                mock.call("/tmp/1_test.html", "w+")]
         mock_open.assert_has_calls(expected_open_calls, any_order=True)
-        mock_plot.plot.assert_called_once_with(results)
+        mock_plot.plot.assert_called_once_with(results, include_libs=False)
 
         mock_open.side_effect().write.assert_called_once_with("html_report")
 
@@ -525,10 +625,10 @@ class TaskCommandsTestCase(test.TestCase):
     @mock.patch("rally.cli.commands.task.cliutils.print_list")
     @mock.patch("rally.cli.commands.task.envutils.get_global",
                 return_value="123456789")
-    @mock.patch("rally.cli.commands.task.objects.Task.list",
+    @mock.patch("rally.cli.commands.task.api.Task.list",
                 return_value=[fakes.FakeTask(uuid="a",
-                                             created_at=date.datetime.now(),
-                                             updated_at=date.datetime.now(),
+                                             created_at=dt.datetime.now(),
+                                             updated_at=dt.datetime.now(),
                                              status="c",
                                              tag="d",
                                              deployment_name="some_name")])
@@ -549,10 +649,10 @@ class TaskCommandsTestCase(test.TestCase):
     @mock.patch("rally.cli.commands.task.cliutils.print_list")
     @mock.patch("rally.cli.commands.task.envutils.get_global",
                 return_value="123456789")
-    @mock.patch("rally.cli.commands.task.objects.Task.list",
+    @mock.patch("rally.cli.commands.task.api.Task.list",
                 return_value=[fakes.FakeTask(uuid="a",
-                                             created_at=date.datetime.now(),
-                                             updated_at=date.datetime.now(),
+                                             created_at=dt.datetime.now(),
+                                             updated_at=dt.datetime.now(),
                                              status="c",
                                              tag="d",
                                              deployment_name="some_name")])
@@ -570,7 +670,7 @@ class TaskCommandsTestCase(test.TestCase):
         self.assertEqual(1, self.task.list(deployment="fake",
                                            status="wrong non existing status"))
 
-    @mock.patch("rally.cli.commands.task.objects.Task.list", return_value=[])
+    @mock.patch("rally.cli.commands.task.api.Task.list", return_value=[])
     def test_list_no_results(self, mock_task_list):
         self.assertIsNone(
             self.task.list(deployment="fake", all_deployments=True))
@@ -606,7 +706,7 @@ class TaskCommandsTestCase(test.TestCase):
         self.assertTrue(mock_api.Task.delete.mock_calls == expected_calls)
 
     @mock.patch("rally.cli.commands.task.cliutils.print_list")
-    @mock.patch("rally.cli.commands.task.objects.Task.get")
+    @mock.patch("rally.cli.commands.task.api.Task.get")
     def test_sla_check(self, mock_task_get, mock_print_list):
         data = [{"key": {"name": "fake_name",
                          "pos": "fake_pos",
@@ -630,38 +730,44 @@ class TaskCommandsTestCase(test.TestCase):
         result = self.task.sla_check(task_id="fake_task_id", tojson=True)
         self.assertEqual(0, result)
 
+    @mock.patch("rally.cli.commands.task.os.path.isfile", return_value=True)
     @mock.patch("rally.api.Task.validate")
     @mock.patch("rally.cli.commands.task.open",
                 side_effect=mock.mock_open(read_data="{\"some\": \"json\"}"),
                 create=True)
-    def test_validate(self, mock_open, mock_task_validate):
+    def test_validate(self, mock_open, mock_task_validate,
+                      mock_os_path_isfile):
         self.task.validate("path_to_config.json", "fake_id")
-        mock_task_validate.assert_called_once_with("fake_id", {"some": "json"})
+        mock_task_validate.assert_called_once_with("fake_id", {"some": "json"},
+                                                   None)
 
+    @mock.patch("rally.cli.commands.task.os.path.isfile", return_value=True)
     @mock.patch("rally.cli.commands.task.TaskCommands._load_task",
                 side_effect=task.FailedToLoadTask)
-    def test_validate_failed_to_load_task(self, mock__load_task):
+    def test_validate_failed_to_load_task(self, mock__load_task,
+                                          mock_os_path_isfile):
         args = mock.MagicMock()
         args_file = mock.MagicMock()
 
-        result = self.task.validate("path_to_task", "fake_id",
+        result = self.task.validate("path_to_task", "fake_deployment_id",
                                     task_args=args, task_args_file=args_file)
         self.assertEqual(1, result)
         mock__load_task.assert_called_once_with(
             "path_to_task", args, args_file)
 
+    @mock.patch("rally.cli.commands.task.os.path.isfile", return_value=True)
     @mock.patch("rally.cli.commands.task.TaskCommands._load_task")
     @mock.patch("rally.api.Task.validate")
-    def test_validate_invalid(self, mock_task_validate, mock__load_task):
-
+    def test_validate_invalid(self, mock_task_validate, mock__load_task,
+                              mock_os_path_isfile):
         mock_task_validate.side_effect = exceptions.InvalidTaskException
         result = self.task.validate("path_to_task", "deployment")
         self.assertEqual(1, result)
         mock_task_validate.assert_called_once_with(
-            "deployment", mock__load_task.return_value)
+            "deployment", mock__load_task.return_value, None)
 
     @mock.patch("rally.common.fileutils._rewrite_env_file")
-    @mock.patch("rally.cli.commands.task.db.task_get", return_value=True)
+    @mock.patch("rally.cli.commands.task.api.Task.get", return_value=True)
     def test_use(self, mock_task_get, mock__rewrite_env_file):
         task_id = "80422553-5774-44bd-98ac-38bd8c7a0feb"
         self.task.use(task_id)
@@ -669,8 +775,96 @@ class TaskCommandsTestCase(test.TestCase):
             os.path.expanduser("~/.rally/globals"),
             ["RALLY_TASK=%s\n" % task_id])
 
-    @mock.patch("rally.cli.commands.task.db.task_get")
+    @mock.patch("rally.cli.commands.task.api.Task.get")
     def test_use_not_found(self, mock_task_get):
         task_id = "ddc3f8ba-082a-496d-b18f-72cdf5c10a14"
         mock_task_get.side_effect = exceptions.TaskNotFound(uuid=task_id)
         self.assertRaises(exceptions.TaskNotFound, self.task.use, task_id)
+
+    @mock.patch("rally.task.exporter.TaskExporter.get")
+    def test_export(self, mock_task_exporter_get):
+        mock_client = mock.Mock()
+        mock_exporter_class = mock.Mock(return_value=mock_client)
+        mock_task_exporter_get.return_value = mock_exporter_class
+        self.task.export("fake_uuid", "file-exporter:///fake_path.json")
+        mock_task_exporter_get.assert_called_once_with("file-exporter")
+        mock_client.export.assert_called_once_with("fake_uuid")
+
+    @mock.patch("rally.task.exporter.TaskExporter.get")
+    def test_export_exception(self, mock_task_exporter_get):
+        mock_client = mock.Mock()
+        mock_exporter_class = mock.Mock(return_value=mock_client)
+        mock_task_exporter_get.return_value = mock_exporter_class
+        mock_client.export.side_effect = IOError
+        self.task.export("fake_uuid", "file-exporter:///fake_path.json")
+        mock_task_exporter_get.assert_called_once_with("file-exporter")
+        mock_client.export.assert_called_once_with("fake_uuid")
+
+    @mock.patch("rally.cli.commands.task.sys.stdout")
+    @mock.patch("rally.task.exporter.TaskExporter.get")
+    def test_export_InvalidConnectionString(self, mock_task_exporter_get,
+                                            mock_stdout):
+        mock_exporter_class = mock.Mock(
+            side_effect=exceptions.InvalidConnectionString)
+        mock_task_exporter_get.return_value = mock_exporter_class
+        self.task.export("fake_uuid", "file-exporter:///fake_path.json")
+        mock_stdout.write.assert_has_calls([
+            mock.call("The connection string is not valid: None. "
+                      "Please check your connection string."),
+            mock.call("\n")])
+        mock_task_exporter_get.assert_called_once_with("file-exporter")
+
+    @mock.patch("rally.cli.commands.task.sys.stdout")
+    @mock.patch("rally.cli.commands.task.api.Task")
+    @ddt.data({"error_type": "test_no_trace_type",
+               "error_message": "no_trace_error_message",
+               "error_traceback": None,
+               },
+              {"error_type": "test_error_type",
+               "error_message": "test_error_message",
+               "error_traceback": "test\nerror\ntraceback",
+               })
+    @ddt.unpack
+    def test_show_task_errors_no_trace(self, mock_task, mock_stdout,
+                                       error_type, error_message,
+                                       error_traceback=None):
+
+        test_uuid = "test_task_id"
+        error_data = [error_type, error_message]
+        if error_traceback:
+            error_data.append(error_traceback)
+        mock_task.get_detailed.return_value = {
+            "id": "task",
+            "uuid": test_uuid,
+            "status": "finished",
+            "results": [{
+                "key": {
+                    "name": "fake_name",
+                    "pos": "fake_pos",
+                    "kw": "fake_kw"
+                },
+                "info": {
+                    "load_duration": 3.2,
+                    "full_duration": 3.5,
+                    "iterations_count": 1,
+                    "iterations_failed": 1,
+                    "atomic": {"foo": {}, "bar": {}}},
+
+                "iterations": [
+                    {"duration": 0.9,
+                     "idle_duration": 0.1,
+                     "output": {"additive": [], "complete": []},
+                     "atomic_actions": {"foo": 0.6, "bar": 0.7},
+                     "error": error_data
+                     },
+                ]},
+            ],
+            "verification_log": json.dumps([error_type, error_message,
+                                            error_traceback])
+        }
+        self.task.detailed(test_uuid)
+        mock_task.get_detailed.assert_called_once_with(test_uuid,
+                                                       extended_results=True)
+        mock_stdout.write.assert_has_calls([
+            mock.call(error_traceback or "No traceback available.")
+        ], any_order=False)
