@@ -25,9 +25,10 @@ class SerialScenarioRunnerTestCase(test.TestCase):
     @mock.patch("rally.task.runner._run_scenario_once")
     def test__run_scenario(self, mock__run_scenario_once):
         times = 5
-        result = {"duration": 10, "idle_duration": 0, "error": [],
-                  "scenario_output": {}, "atomic_actions": {},
-                  "timestamp": 1}
+        result = {"duration": 10., "idle_duration": 0., "error": [],
+                  "output": {"additive": [], "complete": []},
+                  "atomic_actions": {},
+                  "timestamp": 1.}
         mock__run_scenario_once.return_value = result
         expected_results = [[result] for i in range(times)]
 
