@@ -14,6 +14,7 @@
 #    under the License.
 
 from boto import exception as boto_exception
+import ddt
 import mock
 from neutronclient.common import exceptions as neutron_exceptions
 
@@ -375,6 +376,33 @@ class NeutronPortTestCase(test.TestCase):
             raw_res["device_id"], {"port_id": raw_res["id"]})
 
 
+@ddt.ddt
+class NeutronSecurityGroupTestCase(test.TestCase):
+
+    @ddt.data(
+        {"admin": mock.Mock(), "admin_required": True},
+        {"admin": None, "admin_required": False})
+    @ddt.unpack
+    def test_list(self, admin, admin_required):
+        sg_list = [{"tenant_id": "user_tenant", "name": "default"},
+                   {"tenant_id": "user_tenant", "name": "foo_sg"}]
+
+        neut = resources.NeutronSecurityGroup()
+        neut.user = mock.MagicMock()
+        neut._resource = "security_group"
+        neut.tenant_uuid = "user_tenant"
+
+        neut.user.neutron().list_security_groups.return_value = {
+            "security_groups": sg_list
+        }
+
+        expected_result = [sg_list[1]]
+        self.assertEqual(expected_result, list(neut.list()))
+
+        neut.user.neutron().list_security_groups.assert_called_once_with(
+            tenant_id=neut.tenant_uuid)
+
+
 class NeutronQuotaTestCase(test.TestCase):
 
     @mock.patch("%s.NeutronQuota._manager" % BASE)
@@ -391,19 +419,58 @@ class NeutronQuotaTestCase(test.TestCase):
         self.assertEqual("foo", getattr(admin, res._service)())
 
 
+@ddt.ddt
 class GlanceImageTestCase(test.TestCase):
 
-    @mock.patch("%s.GlanceImage._manager" % BASE)
-    def test_list(self, mock_glance_image__manager):
+    @mock.patch("rally.plugins.openstack.wrappers.glance.wrap")
+    def test__wrapper_admin(self, mock_glance_wrap):
+        admin = mock.Mock()
+        glance = resources.GlanceImage(admin=admin)
+        wrapper = glance._wrapper()
+
+        mock_glance_wrap.assert_called_once_with(admin.glance, glance)
+        self.assertEqual(wrapper, mock_glance_wrap.return_value)
+
+    @mock.patch("rally.plugins.openstack.wrappers.glance.wrap")
+    def test__wrapper_user(self, mock_glance_wrap):
+        user = mock.Mock()
+        glance = resources.GlanceImage(user=user)
+        wrapper = glance._wrapper()
+
+        mock_glance_wrap.assert_called_once_with(user.glance, glance)
+        self.assertEqual(wrapper, mock_glance_wrap.return_value)
+
+    @mock.patch("rally.plugins.openstack.wrappers.glance.wrap")
+    def test__wrapper_admin_preferred(self, mock_glance_wrap):
+        admin = mock.Mock()
+        user = mock.Mock()
+        glance = resources.GlanceImage(admin=admin, user=user)
+        wrapper = glance._wrapper()
+
+        mock_glance_wrap.assert_called_once_with(admin.glance, glance)
+        self.assertEqual(wrapper, mock_glance_wrap.return_value)
+
+    def test_list(self):
         glance = resources.GlanceImage()
-        glance.tenant_uuid = mock.MagicMock()
+        glance._wrapper = mock.Mock()
+        glance.tenant_uuid = mock.Mock()
 
-        mock_glance_image__manager().list.return_value = ["a", "b", "c"]
-
-        self.assertEqual(["a", "b", "c"], glance.list())
-        mock_glance_image__manager().list.assert_called_once_with(
+        self.assertEqual(glance.list(),
+                         glance._wrapper.return_value.list_images.return_value)
+        glance._wrapper.return_value.list_images.assert_called_once_with(
             owner=glance.tenant_uuid)
 
+    def test_delete(self):
+        glance = resources.GlanceImage()
+        glance._wrapper = mock.Mock()
+        glance.raw_resource = mock.Mock()
+
+        self.assertEqual(
+            glance.delete(),
+            glance._wrapper.return_value.delete_image.return_value)
+        glance._wrapper.return_value.delete_image.assert_called_once_with(
+            glance.raw_resource)
+
 
 class CeilometerTestCase(test.TestCase):
 
