@@ -190,7 +190,11 @@ class TempestConfigTestCase(test.TestCase):
         for item in expected:
             self.assertIn(item, result)
 
-    def test__configure_network_feature_enabled(self):
+    @ddt.data({}, {"version": "4.1.0", "args": ("extensions", "/extensions"),
+                   "kwargs": {"retrieve_all": True}})
+    @ddt.unpack
+    def test__configure_network_feature_enabled(
+            self, version="4.0.0", args=("/extensions",), kwargs={}):
         self.tempest_conf.available_services = ["neutron"]
         client = self.tempest_conf.clients.neutron()
         client.list_ext.return_value = {
@@ -201,7 +205,9 @@ class TempestConfigTestCase(test.TestCase):
             ]
         }
 
+        mock.patch("neutronclient.version.__version__", version).start()
         self.tempest_conf._configure_network_feature_enabled()
+        client.list_ext.assert_called_once_with(*args, **kwargs)
         self.assertEqual(self.tempest_conf.conf.get(
             "network-feature-enabled", "api_extensions"),
             "dvr,extra_dhcp_opt,extraroute")
@@ -398,22 +404,14 @@ class TempestResourcesContextTestCase(test.TestCase):
 
     @mock.patch("rally.plugins.openstack.wrappers.glance.wrap")
     def test__discover_or_create_image_when_image_exists(self, mock_wrap):
-        client = self.context.clients.glance()
-        client.images.list.return_value = [fakes.FakeImage(name="CirrOS",
-                                                           status="active")]
+        client = mock_wrap.return_value
+        client.list_images.return_value = [fakes.FakeImage(name="CirrOS")]
+
         image = self.context._discover_or_create_image()
         self.assertEqual("CirrOS", image.name)
+        self.assertEqual(0, client.create_image.call_count)
         self.assertEqual(0, len(self.context._created_images))
 
-    # @mock.patch("six.moves.builtins.open")
-    # def test__discover_or_create_image(self, mock_wrap, mock_open):
-    #     client = self.context.clients.glance()
-    #     client.images.create.side_effect = [fakes.FakeImage(id="id1")]
-
-    #     image = self.context._discover_or_create_image()
-    #     self.assertEqual("id1", image.id)
-    #     self.assertEqual("id1", self.context._created_images[0].id)
-
     @mock.patch("rally.plugins.openstack.wrappers.glance.wrap")
     def test__discover_or_create_image(self, mock_wrap):
         client = mock_wrap.return_value
@@ -429,7 +427,7 @@ class TempestResourcesContextTestCase(test.TestCase):
             image_location=mock.ANY,
             disk_format=CONF.image.disk_format,
             name=mock.ANY,
-            is_public=True)
+            visibility="public")
 
     def test__discover_or_create_flavor_when_flavor_exists(self):
         client = self.context.clients.nova()
