@@ -333,13 +333,14 @@ class TempestInstallAndUninstallTestCase(BaseTestCase):
 
 
 class TempestVerifyTestCase(BaseTestCase):
-    def _get_fake_call(self, testr_args):
+    def _get_fake_call(self, testr_args,
+                       concur_args="--parallel --concurrency 0"):
         return (
-            "%(venv)s testr run --subunit --parallel"
-            " --concurrency 0 %(testr_args)s"
+            "%(venv)s testr run --subunit %(concur_args)s %(testr_args)s"
             " | tee %(log_file)s"
             " | %(venv)s subunit-trace -f -n" % {
                 "venv": self.verifier.venv_wrapper,
+                "concur_args": concur_args,
                 "testr_args": testr_args,
                 "log_file": self.verifier.path("subunit.stream")})
 
@@ -359,7 +360,7 @@ class TempestVerifyTestCase(BaseTestCase):
         set_name = "compute"
         fake_call = self._get_fake_call("tempest.api.%s" % set_name)
 
-        self.verifier.verify(set_name, None, None, None, 0)
+        self.verifier.verify(set_name, None, None, None, 0, False)
 
         self.assertEqual(2, mock_tempest_is_configured.call_count)
         mock_tempest_config.assert_called_once_with(self.verifier.deployment)
@@ -389,7 +390,7 @@ class TempestVerifyTestCase(BaseTestCase):
         set_name = "identity"
         fake_call = self._get_fake_call("tempest.api.%s" % set_name)
 
-        self.verifier.verify(set_name, None, None, None, 0)
+        self.verifier.verify(set_name, None, None, None, 0, False)
 
         mock_tempest_is_configured.assert_called_once_with()
         self.assertFalse(mock_tempest_config.called)
@@ -418,7 +419,7 @@ class TempestVerifyTestCase(BaseTestCase):
         fake_call = self._get_fake_call("tempest.api.%s" % set_name)
         mock_subprocess.side_effect = subprocess.CalledProcessError
 
-        self.verifier.verify(set_name, None, None, None, 0)
+        self.verifier.verify(set_name, None, None, None, 0, False)
 
         mock_tempest_is_configured.assert_called_once_with()
         self.assertFalse(mock_tempest_config.called)
@@ -445,7 +446,7 @@ class TempestVerifyTestCase(BaseTestCase):
         tests_file = "/path/to/tests/file"
         fake_call = self._get_fake_call("--load-list %s" % tests_file)
 
-        self.verifier.verify("", None, tests_file, None, 0)
+        self.verifier.verify("", None, tests_file, None, 0, False)
         self.verifier.verification.start_verifying.assert_called_once_with("")
 
         mock_subprocess.check_call.assert_called_once_with(
@@ -453,6 +454,73 @@ class TempestVerifyTestCase(BaseTestCase):
             shell=True)
         mock_tempest_parse_results.assert_called_once_with(None, None)
 
+    @mock.patch(TEMPEST_PATH + ".tempest.Tempest.parse_results",
+                return_value=None)
+    @mock.patch(TEMPEST_PATH + ".tempest.Tempest.env")
+    @mock.patch(TEMPEST_PATH + ".tempest.subprocess")
+    @mock.patch(TEMPEST_PATH + ".config.TempestResourcesContext")
+    @mock.patch(TEMPEST_PATH + ".tempest.Tempest.is_configured",
+                return_value=True)
+    def test_verify_concurrency_equals_to_1(
+            self, mock_tempest_is_configured, mock_tempest_resources_context,
+            mock_subprocess, mock_tempest_env, mock_tempest_parse_results):
+        set_name = "identity"
+        fake_call = self._get_fake_call(
+            "tempest.api.%s" % set_name, "--concurrency 1")
+
+        self.verifier.verify("identity", None, None, None, 1, False)
+        self.verifier.verification.start_verifying.assert_called_once_with(
+            set_name)
+
+        mock_subprocess.check_call.assert_called_once_with(
+            fake_call, env=mock_tempest_env, cwd=self.verifier.path(),
+            shell=True)
+        mock_tempest_parse_results.assert_called_once_with(None, None)
+
+    @mock.patch(TEMPEST_PATH + ".tempest.Tempest.parse_results",
+                return_value=None)
+    @mock.patch(TEMPEST_PATH + ".tempest.Tempest.env")
+    @mock.patch(TEMPEST_PATH + ".tempest.subprocess")
+    @mock.patch(TEMPEST_PATH + ".config.TempestResourcesContext")
+    @mock.patch(TEMPEST_PATH + ".tempest.Tempest.is_configured",
+                return_value=True)
+    def test_verify_concurrency_doesnt_equal_to_1(
+            self, mock_tempest_is_configured, mock_tempest_resources_context,
+            mock_subprocess, mock_tempest_env, mock_tempest_parse_results):
+        set_name = "identity"
+        fake_call = self._get_fake_call("tempest.api.%s" % set_name)
+
+        self.verifier.verify("identity", None, None, None, 0, False)
+        self.verifier.verification.start_verifying.assert_called_once_with(
+            set_name)
+
+        mock_subprocess.check_call.assert_called_once_with(
+            fake_call, env=mock_tempest_env, cwd=self.verifier.path(),
+            shell=True)
+        mock_tempest_parse_results.assert_called_once_with(None, None)
+
+    @mock.patch(TEMPEST_PATH + ".tempest.Tempest.parse_results",
+                return_value=None)
+    @mock.patch(TEMPEST_PATH + ".tempest.Tempest.env")
+    @mock.patch(TEMPEST_PATH + ".tempest.subprocess")
+    @mock.patch(TEMPEST_PATH + ".config.TempestResourcesContext")
+    @mock.patch(TEMPEST_PATH + ".tempest.Tempest.is_configured",
+                return_value=True)
+    def test_verify_run_failed_tests_(self, mock_tempest_is_configured,
+                                      mock_tempest_resources_context,
+                                      mock_subprocess, mock_tempest_env,
+                                      mock_tempest_parse_results):
+        fake_call = self._get_fake_call("--failing")
+        self.verifier.verify("", None, None, None, 0, True)
+
+        self.verifier.verification.start_verifying.assert_called_once_with(
+            "re-run-failed")
+
+        mock_subprocess.check_call.assert_called_once_with(
+            fake_call, env=mock_tempest_env, cwd=self.verifier.path(),
+            shell=True)
+        mock_tempest_parse_results.assert_called_once_with(None, None)
+
     def test_import_results(self):
         set_name = "identity"
         log_file = "log_file"
